/****************************************************************************
*
*     Module Filename:  Krnl_Vis.h
*     Module Name:      Kernel Visibility API
*     Description:      Visibility API for the SimSoup Kernel
*
*	  Author:           Chris Gordon-Smith
*
*     Copyright (C) 2007, 2008 Chris Gordon-Smith
*     Email: See http://www.simsoup.info/
*
*     This program is free software; you can redistribute it and/or modify it
*     under the terms of the GNU General Public License as published by the
*     Free Software Foundation; either version 2 of the License, or (at your
*     option) any later version.
*
*     This program is distributed in the hope that it will be useful, but
*     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
*     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
*     for more details.
*
*     You should have received a copy of the GNU General Public License along
*     with this program; if not, write to the Free Software Foundation, Inc.,
*     59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*
*     See the file Licence.txt for a copy of the GNU General Public License.
*
*      Amendment History:- 
*
*              Version	  Date                        Update Description
*              =======    ===========                 ==================
*
*
*
*
****************************************************************************/

//#include  "Krnl_Enums.h"   // Kernel codes can be used with the Kernel Interface

#ifndef Krnl_Vis_H
#define Krnl_Vis_H

#include  "Kernel_Codes.h"

//      Interaction_Type codes

//        enum Interaction_Type_Enum {Interaction_Type_Enumr__Constr,
//        Interaction_Type_Enumr__Fission,
//        Interaction_Type_Enumr__Transfm};

class   Action_Request;
class   Chemistry;
class   Reactor;
class   Scenario_Manager;
class   Scenario;
class   Interaction_Type;
class   Molecule_Type;
class   Realised_Molecule_Type;
class   Realisable_Interaction_Type;
class   Simulation_Scheduler;

class   Krnl_Vis

{
//===== Relationships with other classes ===================================================================

//==========================================================================================================

    public:

        static  void Initialise_API();

//      Chemistry functions

        class  Chemistry_
        {
            friend  class Krnl_Vis;
            public:
                static  void Clear();
                static  void Initialise();

                static Molecule_Type*  Get_Molecule_Type_Ptr(int Node_Index);
                static int Get_Number_Of_Molecule_Types();

                static  Interaction_Type* Get_Current_Int_Type_Ptr();
                static  bool Iter_To_First_Int_Type();
                static  bool Iter_To_Next_Int_Type();

                static  Molecule_Type* Get_Current_Mol_Type_Ptr();
                static  bool Iter_To_First_Mol_Type();
                static  bool Iter_To_Next_Mol_Type();


            private: 
                static  Chemistry* Chemistry_Ptr;
        };

//      Interaction_Type functions

        class  IntType
        {
            public:
                static std::string  Convert_Int_Type_Categ_To_Text(Interaction_Type_Enum Interaction_Type_Enumr);
                static Interaction_Type_Enum   Get_Category(Interaction_Type*  IntType_Ptr); 

                static double Get_Heat_Release(Interaction_Type* IntType_Ptr);

                static double Get_Forward_Activation_Energy(Interaction_Type* IntType_Ptr);      
                static double Get_Reverse_Activation_Energy(Interaction_Type* IntType_Ptr); 
                static double Get_Activated_Complex_Energy(Interaction_Type* IntType_Ptr); 

                static double  Get_Forward_Rate_Constant(Interaction_Type* IntType_Ptr);
                static double  Get_Reverse_Rate_Constant(Interaction_Type* IntType_Ptr);

                static int   Get_Identifier(Interaction_Type* IntType_Ptr);
                static void  Get_Int_Type_Characteristics(Interaction_Type*  IntType_Ptr,
                                                          int& Interaction_Type_Identifier,
                                                          Interaction_Type_Enum&  Interaction_Type_Enumr,
                                                          int& Reactant1_Identifier,
                                                          int& Reactant2_Identifier,
                                                          int& Product1_Identifier,
                                                          int& Product2_Identifier,
                                                          int& Catalyst_Identifier);

                static void Get_Int_Type_Inputs_No_Dups(Interaction_Type* IntType_Ptr,
                                                        VCSList* TList__Of_Interaction_Inputs_Ptr);

                static void Get_Int_Type_Products_With_Dups(Interaction_Type* IntType_Ptr,
                                                            VCSList* TList__Of_Interaction_Inputs_Ptr);

                static int Get_Product1_MolType_Id(Interaction_Type*  IntType_Ptr);
                static int Get_Product2_MolType_Id(Interaction_Type*  IntType_Ptr);
                static Molecule_Type* Get_Product1_MolType_Ptr(Interaction_Type*  IntType_Ptr);
                static Molecule_Type* Get_Product2_MolType_Ptr(Interaction_Type*  IntType_Ptr);
                static string Get_Product2_MolType_Text(Interaction_Type*  IntType_Ptr);  // Needed when there
                                                                                          // is no Product 2
                static int Get_Reactant1_MolType_Id(Interaction_Type*  IntType_Ptr);
                static int Get_Reactant2_MolType_Id(Interaction_Type*  IntType_Ptr);
                static Molecule_Type* Get_Reactant1_MolType_Ptr(Interaction_Type*  IntType_Ptr);
                static Molecule_Type* Get_Reactant2_MolType_Ptr(Interaction_Type*  IntType_Ptr);
                static string Get_Reactant2_MolType_Text(Interaction_Type*  IntType_Ptr); 
        };

//      Molecule_Type functions

        class  MolType
        {
            public:
                static  int  Get_Identifier(Molecule_Type*  MolType_Ptr);
                static  int  Get_Mass(Molecule_Type*  MolType_Ptr);
                static  int  Get_Max_Mass();
                static  double Get_Potential_Energy(Molecule_Type*  MolType_Ptr);

                static  bool  Iter_To_First_IntType_With_MolType_As_Input(Molecule_Type*  MolType_Ptr);
                static  bool  Iter_To_Next_IntType_With_MolType_As_Input(Molecule_Type*  MolType_Ptr);
                static  Interaction_Type* Get_Current_IntType_With_MolType_As_Input(Molecule_Type*  MolType_Ptr);

                static  bool  Iter_To_First_IntType_With_MolType_As_Product(Molecule_Type*  MolType_Ptr);
                static  bool  Iter_To_Next_IntType_With_MolType_As_Product(Molecule_Type*  MolType_Ptr);
                static  Interaction_Type* Get_Current_IntType_With_MolType_As_Product(Molecule_Type*  MolType_Ptr);
        };
     
//      Reactor functions  
        
        class  Reactor_
        {
            public:
                static  void Clear();
                static  int  Get_Num_Mols();
                static  double Get_Leakage_Proportion();
                static  int  Get_Time();
                static  int  Get_Volume();
                static  void Initialise();
                static  bool Iter_To_First_R_Int_Type();
                static  bool Iter_To_Next_R_Int_Type();
                static  Realisable_Interaction_Type*  Get_Current_R_Int_Type_Ptr();

                static  bool Iter_To_First_R_Mol_Type();
                static  bool Iter_To_Next_R_Mol_Type();
                static  Realised_Molecule_Type*  Get_Current_R_Mol_Type_Ptr();
        };

//      Realisable Interaction Type functions

        class  Realisable_IntType
        {
            public:
                static Interaction_Type*  Get_Interaction_Type_Ptr(Realisable_Interaction_Type*  Realisable_IntType_Ptr);
                static bool Get_Unrealisable(Realisable_Interaction_Type* Realisable_IntType_Ptr);
                static double Update_Interaction_Rates(Realisable_Interaction_Type* Realisable_IntType_Ptr, int Reactor_Volume);
                static double Get_Interaction_Rate(Realisable_Interaction_Type* Realisable_IntType_Ptr);
        };

//      Realised Molecule_Type functions

        class R_MolType
        {
            public:
                static  Molecule_Type*  Get_Molecule_Type_Ptr(Realised_Molecule_Type*  R_MolType_Ptr);
                static  int   Get_Num_Of_Mols_In_Reactor(Realised_Molecule_Type*  R_MolType_Ptr);
        };

//      Scenario functions

        class Scenario_
        {
            public:
                static  bool  Iter_To_First_Action_Request(Scenario*  Scenario_Ptr);
                static  bool  Iter_To_Next_Action_Request(Scenario*  Scenario_Ptr);
                static  Action_Request*  Get_Current_Action_Req_Ptr(Scenario*  Scenario_Ptr);
        
                static  Scenario*  Get_Curr_Scenario_Ptr();
                static  int        Get_Statistics_Collection_Interval();
        };

//      Simulation Scheduler Functions

        class Simulation_Scheduler_
        {
            public:
                static bool SimProc_IsEnabled();
        };

//      Action_Request functions

        class   Action_Request_
        {
            public:
                static string Get_Subject(Action_Request*  Action_Request_Ptr);
//                static int  Get_Number(Action_Request*  Action_Request_Ptr);
                static void Get_Params(Action_Request*  Action_Request_Ptr, list<string>& Params);
                static  int Get_Repeat_Interval(Action_Request*  Action_Request_Ptr);
                static int  Get_Time(Action_Request*  Action_Request_Ptr);
                static string Get_Type(Action_Request*  Action_Request_Ptr);
                static bool Get_Repeat_Req_Flag(Action_Request*  Action_Request_Ptr);
        };

//      Conctruction functions

        class  Construction_
        {
        public:
            static double Get_Max_Rand_Rate_Constant();
        };

//      Fission functions

        class Fission_
        {
        public:
            static double Get_Max_Rand_Rate_Constant();
        };

    private:

        static  Reactor*   Reactor_Ptr;
        static  Scenario_Manager*  Scenario_Manager_Ptr;
        static  Simulation_Scheduler* Sim_Scheduler_Pt_;
       
};
#endif  //Krnl_Vis_H
