/****************************************************************************
*
*	  Header Filename: Reactor.h
*	  Module Name:     Reactor
*	  Description:     Random Chemistry Reactor Class Definition
*                      This Class represents the Reactor, and holds
*                      references to data structures representing the
*                      Reactor contents.
*	  Author:           Chris Gordon-Smith
*
*     Copyright (C) 2003, 2004, 2005, 2006, 2008 Chris Gordon-Smith
*     Email: See http://www.simsoup.info/
*
*     This program is free software; you can redistribute it and/or modify it
*     under the terms of the GNU General Public License Version 2, as published 
*     by the Free Software Foundation.
*
*     This program is distributed in the hope that it will be useful, but
*     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
*     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
*     for more details.
*
*     You should have received a copy of the GNU General Public License along
*     with this program; if not, write to the Free Software Foundation, Inc.,
*     59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*
*     See the file Licence.txt for a copy of the GNU General Public License.
*
*	  Amendment History:-
*
*		Version	   Date	   			Update Description
*		=======    ===========		==================
*
*
*
*
****************************************************************************/

#ifndef Reactor_H
#define Reactor_H

class   Chemistry;
class   Molecule_Type;
class   Interaction_Type;
class   Realised_Molecule_Type;
class   Realisable_Interaction_Type;
class   Realised_Interaction_Type;
class   Form_View_Reactor;
class   VCSList;
class   Event_Log;
class   Indexed_Series;
//class   Manhattan_Distance_Series;
class   Tracker_Manager;
class   Tracker;
class   Realised_Cycle_Type_Manager;

class Reactor

//  Reactor Class Declaration
//  This Class represents the Reactor, and holds references to data structures
//  representing the Reactor contents.

{
    private:
        class Iterators;

//      Friend Classes amd Functions

//    friend class  Reactor::Iterators;

//===== Relationships with other classes ===================================================================
    private:
        VCSList*  TList__Realised_Mol_Types_Ptr;   // List of Realised Molecule Types

        DN::List<Realisable_Interaction_Type*>  List__Realisable_Int_Type_Ptrs;
                                                 //List of Realisable_Interaction_Types
        list<Realised_Interaction_Type*>  List_Of_Realised_Int_Type_Ptrs; // List of Interaction_Types that
                                                                          // are or have been realised       
        Reactor::Iterators*   Iterators__Ptr;   // Reactor's Iterators object
        
        Indexed_Series*    Num_Molecules_Series_Ptr;
        Indexed_Series*    Num_Ints_Per_Timestep_Series_Ptr;
        Indexed_Series*    Num_MolTypes_Series_Ptr;
        Indexed_Series*    Num_R_Int_Types_Series_Ptr;
        Indexed_Series*    Fissions_Per_Timestep_Series_Ptr;  
        Indexed_Series*    Constructions_Per_Timestep_Series_Ptr;  
        Indexed_Series*    Transformations_Per_Timestep_Series_Ptr;  
        Indexed_Series*    Int_Damping_Series_Ptr;
        Indexed_Series*    Realised_Cycle_Type_Series_Ptr;
        Indexed_Series*    Num_Active_Cycle_Type_Series_Ptr;
        Indexed_Series*    Avg_Active_Cycle_Type_Length_Series_Ptr;
        Indexed_Series*    Avg_Active_Cycle_Rate_Series_Ptr;
        Indexed_Series*    Avg_Active_Cycle_Contrib_Int_Rate_Series_Ptr;
        Indexed_Series*    Min_Active_Cycle_Length_Series_Ptr;
        Indexed_Series*    Max_Active_Cycle_Length_Series_Ptr;

        Indexed_Series*    Heat_Release_Series_Ptr;

        struct Ind_Ser_List_Names   // The Reactor has Indexed Series organised in lists. 
                                   // These are the list names
        {
            string Reactor_Overview;
            string Mol_Counts;
            string Int_Counts;
            string Manhattan_Distances;
        }   Ind_Ser_List_Names_;

                
        enum Ind_Ser_List_Name_Codes
        {
            Reactor_Overview_Cd,
            Mol_Counts_Cd,
            Int_Counts_Cd,
            Manhattan_Dist_Cd
        };

        struct Ind_Ser_Names   // Indexed Series Names: Standard Series 
        {
            string Mols;
            string Realisable_IntTypes;
            string IntRate;
            string MolTypes;
            string Constr_Rate;
            string Fission_Rate;
            string Transfm_Rate;
            string Damped_IntRate;
            string Realsd_Cycle_Types;
            string Num_Actv_Cycl_Types;
            string Avg_Actv_Cycl_Type_Len;
            string Avg_Actv_Cycl_Rate;
            string Avg_Actv_Cycl_Contrib_IntRate;
            string Actv_Cycl_Min_Len;
            string Actv_Cycl_Max_Len;
            string Heat_Release;
        }   Ind_Ser_Names_;

        struct Ind_Ser_Name_Prefixes   //  Prefixes for automatically generated series Names
        {
            string Manhattan;
            string IntRate;
            string MolCount;
        }   Ind_Ser_Names_Prefixes_;

       struct Ind_Ser_Idx_Names   // Names for Indexes in Indexed Series
       {
            string Time;
       }  Ind_Ser_Idx_Names_;

        Tracker_Manager*   Tracker_Manager_Ptr;
        Realised_Cycle_Type_Manager*       RCT_Manager_Ptr;
        
        struct Maint_MolType_Specifier 
        {
            Molecule_Type* MolType_Ptr;
            enum MolType_Maint_Strat {Fixed_Inflow, Fixed_Number};
            MolType_Maint_Strat MolType_Maintenance_Strategy;
            uint Parameter;
            double  Tracker_Probability;
            int     Molecule_Type_Index;
        };
        list<Maint_MolType_Specifier*> List_Of_Maint_MolType_Specs;

       struct Run_Summary_Statistics
        {
            double Max_Interaction_Damping_Rate;
            int Max_Num_Molecules_P_I_Transient;  //Post Initial Transient
            int Max_MolTypes_P_I_Transient;
            int Max_R_Int_Types_P_I_Transient;
            double Max_Int_Rate_P_I_Transient;

            int Min_Num_Molecules_P_I_Transient;  //Post Initial Transient
            int Min_MolTypes_P_I_Transient;
            int Min_R_Int_Types_P_I_Transient;
            double Min_Int_Rate_P_I_Transient;

            int Max_Num_Molecules;
            int Max_MolTypes;
            int Max_R_Int_Types;
            double Max_Int_Rate;

            int Min_Num_Molecules;
            int Min_MolTypes;
            int Min_R_Int_Types;
            double Min_Int_Rate;

            double Sigma_Molecules_P_I_T;   // After the initial transient, incremented by Num Molecules 
            double Sigma_MolTypes_P_I_T;    // * Stats_Collection_Interval each time statistics are collected  
            double Sigma_R_Int_Types_P_I_T; // Used for calculation of average
            double Sigma_Int_Rate_P_I_T;

            int Summation_Index;     // Counts number of times the 'Sigma' values have been incremented.
                                     // Used for calculation of averages
        };
        Run_Summary_Statistics Run_Summary;

//==========================================================================================================

//      Helper Classes

    private:
        class Iterators
        {
            private:
//===== Relationships with other classes ===================================================================

                Reactor*    Reactor__Ptr;
//==========================================================================================================
                int     Iter_Realised_Mol_Types;           // Iterator for Realised Molecule Types


                DN::Iter<Realisable_Interaction_Type*>* Iter__R_Int_Type_Ptrs_Ptr;
                                                           // Iterator for R Int Types

            public:
                Iterators(Reactor* Creator_Ptr);

                bool    Iter_To_First_R_Int_Type() const;
                bool    Iter_To_Next_R_Int_Type() const;
                bool    Iter_To_First_R_Mol_Type();
                bool    Iter_To_Next_R_Mol_Type();
                bool    R_Int_Types_Exist();

                Realisable_Interaction_Type* Get_Current_R_Int_Type_Ptr() const;
                Realised_Molecule_Type*      Get_Current_R_Mol_Type_Ptr() const;
        };
//==========================================================================================================

    public:
//      Public Functions

        void    Initialise();
        Tracker_Manager*  Get_Tracker_Manager_Ptr() {return  Tracker_Manager_Ptr;};
        list<Tracker*>*     Remove_Molecule_Of_Type(Molecule_Type*  Molecule_Type__Ptr);
        bool    Add_Maintained_MolType_Spec(int MolType_Index, int Num_Mols_Fixd, double Tracker_Probability);
        bool    Remove_Maintained_MolType_Spec(int& MolType_Index);
        static void    Add_Mol_Of_Type(Molecule_Type*  MolType_Ptr, 
                                       double  Tracker_Probability);
        static void    Add_Mol_Of_Type(Molecule_Type*  MolType_Ptr, 
                                       list<Tracker*>* Tracker_List_Ptr);
        static void    Add_Mol_Of_Type(Molecule_Type*  MolType_Ptr);
        static void    Add_Mol_Of_Type(Molecule_Type*  MolType_Ptr, 
                                       double  Tracker_Probability, 
                                       list<Tracker*>* Tracker_List_Ptr);

        Realised_Interaction_Type*   Add_Realised_Int_Type(Interaction_Type* Int_Type_Ptr);
        void    Clear();
        void    Disable();
        void    Empty();
        static void Make();
        double  Get_Fitness();
//        string  Get_Name_For_Ind_Ser_List(Ind_Ser_List_Name_Codes Code);
        int     Get_Num_Mols_In_Reactor() const;
        int     Get_Num_Mol_Types_In_Reactor() const;
        int     Get_Num_R_Int_Types_In_Reactor() const;
        double  Get_Reactor_Leakage_Proportion() {return Reactor_Leakage_Proportion;};
        static  list<string> Get_And_Clear_Reactor_Alerts();
        int     Get_Reactor_Time() const;
        static  int  Time () {return Reactor__Ptr->Reactor_Time;};
        int     Get_Reactor_Volume() const;
        void    Set_Reactor_Volume(int Volume) {Reactor_Volume = Volume;};
        void    Get_Run_Summary_Statistics(double& Max_Interaction_Damping_Rate,
                                           int& Max_MolTypes,
                                           int& Max_Num_Molecules,
                                           int& Max_R_Int_Types,
                                           double& Max_Int_Rate,

                                           int& Min_MolTypes,
                                           int& Min_Num_Molecules,
                                           int& Min_R_Int_Types,
                                           double& Min_Int_Rate,

                                           int& Max_MolTypes_P_I_Transient,
                                           int& Max_Num_Molecules_P_I_Transient,
                                           int& Max_R_Int_Types_P_I_Transient,
                                           double& Max_Int_Rate_P_I_Transient,

                                           int& Min_MolTypes_P_I_Transient,
                                           int& Min_Num_Molecules_P_I_Transient,
                                           int& Min_R_Int_Types_P_I_Transient,
                                           double& Min_Int_Rate_P_I_Transient,

                                           double& Avg_Num_Molecules_P_I_T,
                                           double& Avg_Num_MolTypes_P_I_T,
                                           double& Avg_Num_R_Int_Types_P_I_T,
                                           double& Avg_Int_Rate_P_I_T,
                                           int&  Init_Transient_Time);

        Reactor::Iterators*  Get_Iterators_Ptr() const  {return Iterators__Ptr;};
        bool    Is_Empty();
        bool    Is_Enabled()  {return Enabled;};
        bool    MolType_Is_In_Reactor(Molecule_Type*  MolType_Ptr)  const;
        void    Notify_Removed_Int_Type(Interaction_Type*  Int_Type_Ptr);
        void    Notify_New_Int_Type(Interaction_Type*  Int_Type_Ptr);
        static Reactor*   Get_Reactor__Ptr();
        
        Reactor();
        bool    Single_Timestep(int Stats_Collection_Interval);

    private:

//      Private Class data

        static  Reactor*  Reactor__Ptr;  // Pointer to the Reactor data structure

        bool Enabled;  // True if Reactor is Enabled
        int  Number_Of_Molecules_In_Reactor;
        double  Timestep;
        int  Reactor_Time;
        int  Reactor_Volume;                     // Volume of the Reactor
        double  Reactor_Leakage_Proportion;
        
        struct Alert_Flags_Tp   
        {
            bool  Interaction_Damping_Has_Occurred;
            bool  Interaction_Rate_Overflow_Avoidance_Has_Started;
        };
        Alert_Flags_Tp  Alert_Flags;  // Flags recording whether various kinds of Alert have occurred in the Reactor

        int Total_Ints_In_Collection_Int;
        int Total_Constructions_In_Collection_Int;
        int Total_Fissions_In_Collection_Int;
        int Total_Transformations_In_Collection_Int;
        int Total_Ints_Damped_In_Collection_Int;
        double  Total_Heat_Release_In_Collection_Int;
        double  Heat_Release_In_Timestep;
        list<double> Recent_Heat_Release;  
        list<string> Reactor_Alerts;

        int Init_Transient_Time;

//      Private Functions

       Realised_Molecule_Type*   Add_New_Realised_Mol_Type(Molecule_Type* Molecule_Type__Ptr);

       void     Add_Maintained_Molecules();
       void     Add_R_Int_Type(Interaction_Type*  Int_Type_Ptr);
       void     Add_R_Int_Types_For_New_R_MolTyp(Realised_Molecule_Type*  New_Realised_Molecule_Type__Ptr);
       void     Add_R_Int_Type_Products(Realisable_Interaction_Type* R_Int_Type_Ptr,  list<Tracker*>* Tracker_List_Ptr);
       double   Avoid_Interaction_Rate_Overflow(Realisable_Interaction_Type* R_Int_Type_Ptr,
                                                double Basic_Int_Rate,
                                                bool& Overflow_Close_Flag);

       int      Avoid_Interaction_Overshoot(Realisable_Interaction_Type* R_Int_Type_Ptr,
                                            int Num_Ints_Undamped,
                                            bool& Damping_Flag);

       int      Calc_Num_Ints_In_Timestep(Realisable_Interaction_Type* R_Int_Type_Ptr,
                                          int& Num_Ints_Undamped,
                                          bool& Alert_Flag);
       void     Collect_Statistics(int Stats_Collection_Interval);
       void     Execute_Interaction(Realisable_Interaction_Type* R_Int_Type_Ptr);
       void     Healthcheck();

       bool     Is_Int_Type_Realisable(Interaction_Type*  Int_Type_Ptr) const;

       void     Link_R_IntType_To_Inp_R_MolTypes(Realisable_Interaction_Type*
                                                 Realisable_Interaction_Type__Ptr);
       void     Unlink_R_IntType_From_Inp_R_MolTypes(Realisable_Interaction_Type*
                                                     R_Int_Type__Ptr);
                                                 
       void     Link_R_IntType_To_Reactor_Objs(Realisable_Interaction_Type*
                                               Realisable_Interaction_Type__Ptr);
       void     Reactor_Alert(string Alert_Text);
       void     Remove_R_Int_Type(Realisable_Interaction_Type*  Realisable_Interaction_Type__Ptr);

       list<Tracker*>*  Remove_R_Int_Type_Reactants(Realisable_Interaction_Type*
                                                   Realisable_Interaction_Type__Ptr);
       void     Remove_Trackers_And_List(list<Tracker*>* Tracker_List_Ptr);
       void     Remove_Unrealisable_R_Int_Types();
       void     Remove_Outflow_Molecules();

       void     Rem_R_Int_Types_For_Old_R_MolTyp(Realised_Molecule_Type*  New_Realised_Molecule_Type__Ptr);
       void     Update_Fitness();

};

#endif  //Reactor_H
