/****************************************************************************
*
*	  Header Filename:  Scenario.h
*	  Module Name:      Scenario
*	  Description:      Specifies the conditions for a simulation run, including
*                       initial setup and Actions to be triggered externally during the run
*	  Author:           Chris Gordon-Smith
*
*     Copyright (C) 2005, 2006, 2008 Chris Gordon-Smith
*     Email: See http://www.simsoup.info/
*
*     This program is free software; you can redistribute it and/or modify it
*     under the terms of the GNU General Public License Version 2, as published 
*     by the Free Software Foundation.
*
*     This program is distributed in the hope that it will be useful, but
*     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
*     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
*     for more details.
*
*     You should have received a copy of the GNU General Public License along
*     with this program; if not, write to the Free Software Foundation, Inc.,
*     59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*
*     See the file Licence.txt for a copy of the GNU General Public License.
*
*      Amendment History:- 
*
*              Version	  Date                        Update Description
*              =======    ===========                 ==================
*
*
*
*
****************************************************************************/

#ifndef Scenario_H
#define Scenario_H

class Action_Request;

class   Scenario
{
//===== Relationships with other classes ===================================================================
 
//==========================================================================================================

    public:
        enum    Execution_State {New, Executing, Executed};
                Scenario();
                Scenario(string  Scenario_Name);
                Scenario(int Max_Mass_For_Scen,
                        int Num_MolTypes,          
                        int Num_Maint_MolTypes,   
                        double Max_Rate_Const_Construction,  
                        double Max_Rate_Const_Fission,      
                        int Num_Cat_Trans,        
                        int Random_Seed, 
                        int End_Time,      
                        double  Trackers_Per_Maint_Mol,        
                        int Stats_Collection_Interval,
                        string Name);

        void    Setup(int Max_Mass_For_Scen,
                      int Num_MolTypes,          
                      int Num_Maint_MolTypes,   
                      double Max_Rate_Const_Construction,  
                      double Max_Rate_Const_Fission,      
                      int Num_Cat_Trans,        
                      int Random_Seed, 
                      int End_Time,            
                      double  Trackers_Per_Maint_Mol,  
                      int Stats_Collection_Interval,
                      string Name);
      
        void    Add_ActReq_To_Scen(Action_Request*  Action_Req_Pt);
        void    Schedule_ActReq(Action_Request*  Action_Req_Pt);
        void    Clear();
        Action_Request* Get_Curr_Scen_ActReq_Pt();
        bool    Iter_First_Scen_ActReq();
        bool    Iter_Next_Scen_ActReq();
        bool    Run(int Num_Timesteps);
        Execution_State Get_Execution_State();
        void    Set_Statistics_Collection_Interval(int Interval) {Statistics_Collection_Interval = Interval;}
        int     Get_Statistics_Collection_Interval() {return Statistics_Collection_Interval;}

    private:
        class Compare_ActReq_Time
        {
            public:
                bool operator() (const Action_Request* x, const Action_Request* y);
        };

        list <Action_Request*> Scenario_ActReq_Ls;  // Action Requests setup by user
                                                    // to define the scanario
        list <Action_Request*>::iterator  Curr_Scen_ActReq_It;    
        priority_queue<Action_Request*,
                       vector<Action_Request*>,
                       Compare_ActReq_Time> 
                            ActReq_Sched_Ls;        // Scheduled Action Requests
        int Max_Mass_For_Scen;   // Maximum mass for Molecule Types
        int Random_Seed;
        int Random_Seed_2;
        string Name;
        int Num_MolTypes;    
        int Num_Maint_MolTypes;
        double Max_Rate_Const_Construction;
        double Max_Rate_Const_Fission;
        int Num_Cat_Trans;      
        int End_Time;      
        double  Trackers_Per_Maint_Mol;
        int Stats_Collection_Interval;

        list<string> Scenario_Setup_Log_Outputs;
        void  Display_Simulation();
        void  Handle_Reactor_Alerts();
        int Statistics_Collection_Interval;  
//        void  Output_Summary_Statistics();
        void  Output_Warning(string Warning_Messsage);
        void  Log_ActReq_Exec_Outcome_If_Reqd(Action_Request*  ActReq_Pt) const;
        bool  Schedule_Repeat_Exec_If_Reqd(Action_Request*  ActReq_Pt);                      
//        Action_Request*  Get_Next_Sched_ActReq_Pt();

        Execution_State Exec_State;
};
#endif  //Scenario_H
