/****************************************************************************
*
*	  Module Filename:  Simulation_Input.cpp
*	  Module Name:      Simulation_Input
*     Description:      Simulation Input Manager
*                      
*     Author:           Chris Gordon-Smith
*
*     Copyright (C) 2008 Chris Gordon-Smith
*     Email: See http://www.simsoup.info/
*
*     This program is free software; you can redistribute it and/or modify it
*     under the terms of the GNU General Public License as published by the
*     Free Software Foundation; either version 2 of the License, or (at your
*     option) any later version.
*
*     This program is distributed in the hope that it will be useful, but
*     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
*     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
*     for more details.
*
*     You should have received a copy of the GNU General Public License along
*     with this program; if not, write to the Free Software Foundation, Inc.,
*     59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*
*     See the file Licence.txt for a copy of the GNU General Public License.
*     Amendment History:-
*
*         Version    Date                 Update Description
*         =======    ===========          ==================
*
*
*
*
****************************************************************************/

#include  "globldef.h"
#include  "Library_Headers.h"

//--------------  Persistent Data Manager Subsystem  -----------------------------------------------------------
#include  "Simulation_Input.h"
#include  "Input_Record.h"
//--------------------------------------------------------------------------------------------------------------

Simulation_Input*  Simulation_Input::Instance_Ptr = 0;
//----------------------------------------------------------------------------------------------------------------


bool  Simulation_Input::Read_Input_Records_And_Validate_Syntax(string& PathName, 
                                                               list<Input_Record*>* Input_Records_Ptr, 
                                                               string& Error_Tx)

//  Read the input file into the specified list of strings (Input_Lines)
//  Each word (token) is a separate string in an Input_Line.
//  Each complete Input_Line is added to the Input_Lines

{
    TRACE("Simulation_Input::Read_Input_Records_And_Validate_Syntax");
    filebuf  InBuf;
    istream Input_Stream(&InBuf);
    InBuf.open(PathName.c_str(),ios::in);
    bool EOF_Flag = false;
    bool Record_OK;
    for(int Record_Num = 1; ; Record_Num++)
    BEGIN
        CREATE_OBJECT(Input_Record, Input_Record*  Rec_Ptr = new Input_Record());
        Record_OK = Rec_Ptr->Read_Record_And_Validate_Syntax(Input_Stream, Record_Num, EOF_Flag, Error_Tx);    
        if (not EOF_Flag) Rec_Ptr->Print(Record_Num);
        if (Record_OK)
        THEN
            Input_Records_Ptr->push_back(Rec_Ptr);
            if (EOF_Flag) return true;
        ELSE
            // Output the record with the error message
            Error_Tx = string(" Syntax error in record ")
                     + To_String(Record_Num)
                     + string(" - ")
                     + Rec_Ptr->Get_Record_As_String() 
                     + string("\n")
                     + Error_Tx; 
            return false;
        ENDIF
    END
}
//-----------------------------------------------------------------------------------------------------------------

bool Simulation_Input::Text_To_Record_Type(string  Rcd_Type_Text, Record_Type_Code&  Rcd_Type, string& Error_Text)

//  Get the Line Type Code for the specified Line Type Text

{
    if (Record_Type_Map.find(Rcd_Type_Text) not_eq Record_Type_Map.end())
    THEN
        Rcd_Type = Record_Type_Map[Rcd_Type_Text];
        return true;
    ELSE
        Error_Text = "Invalid Input Record Type " + String_In_Quotes(Rcd_Type_Text);
        return false;
    ENDIF
}

//-----------------------------------------------------------------------------------------------------------------
Simulation_Input::Simulation_Input()

{
    TRACE("Simulation_Input::Simulation_Input");
    Instance_Ptr = this;

//  Record Type Texts and Codes

//    Record_Type_Map[""] = Blank_Line_Cd;

    Record_Type_Map["Add_Molecule_Req"] = Add_Mol_Req_Rcd_Cd;
    Record_Type_Map["Add_MolType_Req"] = Add_MolType_Req_Rcd_Cd;

//    Record_Type_Map["Add_IntType_Req"] = Add_IntType_Req_Rcd_Cd;
    Record_Type_Map["Add_Constr_Req"] = Add_Constr_Req_Rcd_Cd;
    Record_Type_Map["Add_Fission_Req"] = Add_Fission_Req_Rcd_Cd;
    Record_Type_Map["Add_Transfm_Req"] = Add_Transfm_Req_Rcd_Cd;
    Record_Type_Map["ChemEvol_Req"] = ChemEvol_Req_Rcd_Cd;
    Record_Type_Map["//"] = Comment_Line_Cd;
}
