/****************************************************************************
*
*     Module Filename:  GUI_Presentation.cpp
*     Module Name:      GUI_Presentation
*     Description:      Implementation of the SimSoup Presentation Layer
*                      
*     Author:           Chris Gordon-Smith
*
*     Copyright (C) 2007 Chris Gordon-Smith
*     Email: See http://www.simsoup.info/
*
*     This program is free software; you can redistribute it and/or modify it
*     under the terms of the GNU General Public License as published by the
*     Free Software Foundation; either version 2 of the License, or (at your
*     option) any later version.
*
*     This program is distributed in the hope that it will be useful, but
*     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
*     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
*     for more details.
*
*     You should have received a copy of the GNU General Public License along
*     with this program; if not, write to the Free Software Foundation, Inc.,
*     59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*
*     See the file Licence.txt for a copy of the GNU General Public License.
*     Amendment History:-
*
*         Version    Date                 Update Description
*         =======    ===========          ==================
*
*
*
*
****************************************************************************/

//#define  ENABLE_TRACE_FILE  

#include    "globldef.h"
#include    "Library_Headers.h"

#include     <gtkmm.h>

//--------------  GUI_Presentation  Subsystem  --------------------------------------------------------------------
#include    "GUI_Presentation_API.h"
#include    "GUI_Presentation.h"
#include    "Drawing_Area.h"
#include    "Colour.h"
//----------------------------------------------------------------------------------------------------------------

//--------------  User Interface Manager   ----------------------------------------------------------------------
#include    "User_Actions_API.h"
//----------------------------------------------------------------------------------------------------------------


void GUI_Presentation::Add_Row(Widget_Ids::ListStore_Id Id, 
                               list<string>& List_Of_Col_Texts)
{
    TRACE("GUI_Presentation::Add_Row(Widget_Ids::ListStore_Id Id, list<string>& List_Of_Col_Texts)");

    Gtk::TreeModel::Row row =  *(List_Of_ListStores[Id]->append());

    switch (Id)
    BEGIN
       case  Widget_Ids::SimStats_SerList_ListStore:
            if (not List_Of_Col_Texts.empty())
            THEN
                row[SimStats_ListStore_SerList_Cols.SerList_Name_Col] = List_Of_Col_Texts.front();
                List_Of_Col_Texts.pop_front();
            ENDIF
            if (not List_Of_Col_Texts.empty())
            THEN
                row[SimStats_ListStore_SerList_Cols.Display_Flag_Col] = List_Of_Col_Texts.front();
                List_Of_Col_Texts.pop_front();
            ENDIF

            Assert(List_Of_Col_Texts.empty(), "GUI_Presentation::Add_Row - Too many texts for SimStats_SerList_ListStore");
            break;

       default:
            Assert(false, "GUI_Presentation::Add_Row(Widget_Ids::ListStore_Id Id, list<string>& List_Of_Col_Texts) - Invalid ListStore_Id");
            break;
    END
}
//----------------------------------------------------------------------------------------------------------------

void GUI_Presentation::Add_Row(Widget_Ids::ListStore_Id Id, 
                               list<string>& List_Of_Col_Texts, 
                               void* Data_Ptr)

//  Add a row to the specified ListStore. Include the pointer to Linked Data

{
    TRACE("GUI_Presentation::Add_Row");

    Gtk::TreeModel::Row row =  *(List_Of_ListStores[Id]->append());

    switch (Id)
    {
        case Widget_Ids::MolTypes_ListStore:
            if (not List_Of_Col_Texts.empty())
            THEN
                row[MolType_ListStore_Cols.MolType_Id_Col] = List_Of_Col_Texts.front();
                List_Of_Col_Texts.pop_front();
            ENDIF
            if (not List_Of_Col_Texts.empty())
            THEN
                row[MolType_ListStore_Cols.Mass_Col] = List_Of_Col_Texts.front();
                List_Of_Col_Texts.pop_front();
            ENDIF
            if (not List_Of_Col_Texts.empty())
            THEN
                row[MolType_ListStore_Cols.Potential_Energy_Col] = List_Of_Col_Texts.front();
                List_Of_Col_Texts.pop_front();
            ENDIF
            Assert(List_Of_Col_Texts.empty(), "GUI_Presentation::Add_Row - Too many texts for MolTypes_ListStore");
            
            row[MolType_ListStore_Cols.MolType_Ptr_Col] = Data_Ptr;  // Linked data
            break;
//----------------------------------------------------------------------------------------------------------------

       case Widget_Ids::IntTypes_ListStore:
            if (not List_Of_Col_Texts.empty())
            THEN
                row[IntType_ListStore_Cols.IntType_Id_Col] = List_Of_Col_Texts.front();
                List_Of_Col_Texts.pop_front();
            ENDIF
            if (not List_Of_Col_Texts.empty())
            THEN
                row[IntType_ListStore_Cols.Category_Col] = List_Of_Col_Texts.front();
                List_Of_Col_Texts.pop_front();
            ENDIF
            if (not List_Of_Col_Texts.empty())
            THEN
                row[IntType_ListStore_Cols.Heat_Col] = List_Of_Col_Texts.front();
                List_Of_Col_Texts.pop_front();
            ENDIF
            if (not List_Of_Col_Texts.empty())
            THEN
                row[IntType_ListStore_Cols.Fwd_Actn_Energy_Col] = List_Of_Col_Texts.front();
                List_Of_Col_Texts.pop_front();
            ENDIF
            if (not List_Of_Col_Texts.empty())
            THEN
                row[IntType_ListStore_Cols.Rev_Actn_Energy_Col] = List_Of_Col_Texts.front();
                List_Of_Col_Texts.pop_front();
            ENDIF
            if (not List_Of_Col_Texts.empty())
            THEN
                row[IntType_ListStore_Cols.Actv_Cmplx_Energy_Col] = List_Of_Col_Texts.front();
                List_Of_Col_Texts.pop_front();
            ENDIF
            if (not List_Of_Col_Texts.empty())
            THEN
                row[IntType_ListStore_Cols.Fwd_Rate_Const_Col] = List_Of_Col_Texts.front();
                List_Of_Col_Texts.pop_front();
            ENDIF
            if (not List_Of_Col_Texts.empty())
            THEN
                row[IntType_ListStore_Cols.Rev_Rate_Const_Col] = List_Of_Col_Texts.front();
                List_Of_Col_Texts.pop_front();
            ENDIF
            if (not List_Of_Col_Texts.empty())
            THEN
                row[IntType_ListStore_Cols.Reactant1_Col] = List_Of_Col_Texts.front();
                List_Of_Col_Texts.pop_front();
            ENDIF
            if (not List_Of_Col_Texts.empty())
            THEN
                row[IntType_ListStore_Cols.Reactant2_Col] = List_Of_Col_Texts.front();
                List_Of_Col_Texts.pop_front();
            ENDIF
            if (not List_Of_Col_Texts.empty())
            THEN
                row[IntType_ListStore_Cols.Product1_Col] = List_Of_Col_Texts.front();
                List_Of_Col_Texts.pop_front();
            ENDIF
            if (not List_Of_Col_Texts.empty())
            THEN
                row[IntType_ListStore_Cols.Product2_Col] = List_Of_Col_Texts.front();
                List_Of_Col_Texts.pop_front();
            ENDIF

            Assert(List_Of_Col_Texts.empty(), "GUI_Presentation::Add_Row - Too many texts for IntTypes_ListStore");

            row[IntType_ListStore_Cols.IntType_Ptr_Col] = Data_Ptr;  // Linked data
            break;
//----------------------------------------------------------------------------------------------------------------

       case Widget_Ids::R_MolType_ListStore:
            if (not List_Of_Col_Texts.empty())
            THEN
                row[R_MolType_ListStore_Cols.R_MolType_Id_Col] = List_Of_Col_Texts.front();
                List_Of_Col_Texts.pop_front();
            ENDIF
            if (not List_Of_Col_Texts.empty())
            THEN
                row[R_MolType_ListStore_Cols.Concentration_Col] = List_Of_Col_Texts.front();
                List_Of_Col_Texts.pop_front();
            ENDIF

            Assert(List_Of_Col_Texts.empty(), "GUI_Presentation::Add_Row - Too many texts for R_MolType_ListStore");
            
            row[R_MolType_ListStore_Cols.R_MolType_Ptr_Col] = Data_Ptr;  // Linked data
            break;
//----------------------------------------------------------------------------------------------------------------

       case Widget_Ids::R_IntType_ListStore:
            if (not List_Of_Col_Texts.empty())
            THEN
                row[R_IntType_ListStore_Cols.R_IntType_Id_Col] = List_Of_Col_Texts.front();
                List_Of_Col_Texts.pop_front();
            ENDIF
            if (not List_Of_Col_Texts.empty())
            THEN
                row[R_IntType_ListStore_Cols.Category_Col] = List_Of_Col_Texts.front();
                List_Of_Col_Texts.pop_front();
            ENDIF
            if (not List_Of_Col_Texts.empty())
            THEN
                row[R_IntType_ListStore_Cols.Rate_Col] = List_Of_Col_Texts.front();
                List_Of_Col_Texts.pop_front();
            ENDIF
            if (not List_Of_Col_Texts.empty())
            THEN
                row[R_IntType_ListStore_Cols.Reactant1_Col] = List_Of_Col_Texts.front();
                List_Of_Col_Texts.pop_front();
            ENDIF
            if (not List_Of_Col_Texts.empty())
            THEN
                row[R_IntType_ListStore_Cols.Reactant2_Col] = List_Of_Col_Texts.front();
                List_Of_Col_Texts.pop_front();
            ENDIF
            if (not List_Of_Col_Texts.empty())
            THEN
                row[R_IntType_ListStore_Cols.Product1_Col] = List_Of_Col_Texts.front();
                List_Of_Col_Texts.pop_front();
            ENDIF
            if (not List_Of_Col_Texts.empty())
            THEN
                row[R_IntType_ListStore_Cols.Product2_Col] = List_Of_Col_Texts.front();
                List_Of_Col_Texts.pop_front();
            ENDIF

            Assert(List_Of_Col_Texts.empty(), "GUI_Presentation::Add_Row - Too many texts for R_IntType_ListStore");

            row[R_IntType_ListStore_Cols.R_IntType_Ptr_Col] = Data_Ptr;  // Linked data
            break;
//----------------------------------------------------------------------------------------------------------------


        case Widget_Ids::SimStats_Groups_ListStore:
            if (not List_Of_Col_Texts.empty())
            THEN
                row[SimStats_ListStore_Groups_Cols.Group_Name_Col] = List_Of_Col_Texts.front();
                List_Of_Col_Texts.pop_front();
            ENDIF
            if (not List_Of_Col_Texts.empty())
            THEN
                row[SimStats_ListStore_Groups_Cols.Display_Flag_Col] = List_Of_Col_Texts.front();
                List_Of_Col_Texts.pop_front();
            ENDIF
            Assert(List_Of_Col_Texts.empty(), "GUI_Presentation::Add_Row - Too many texts for SimStats_Groups_ListStore");

            row[SimStats_ListStore_Groups_Cols.Display_Group_Spec_Ptr_Col] = Data_Ptr;  // Linked data
            break;
//----------------------------------------------------------------------------------------------------------------

        case Widget_Ids::SimStats_Series_ListStore:
            if (not List_Of_Col_Texts.empty())
            THEN
                row[SimStats_ListStore_Series_Cols.Series_Name_Col] = List_Of_Col_Texts.front();
                List_Of_Col_Texts.pop_front();
            ENDIF
            if (not List_Of_Col_Texts.empty())
            THEN
                row[SimStats_ListStore_Series_Cols.Plot_Flag_Col] = List_Of_Col_Texts.front();
                List_Of_Col_Texts.pop_front();
            ENDIF
            Assert(List_Of_Col_Texts.empty(), "GUI_Presentation::Add_Row - Too many texts for SimStats_Series_ListStore");

            row[SimStats_ListStore_Series_Cols.Indexed_Series_Ptr_Col] = Data_Ptr;  // Linked data
            break;
//----------------------------------------------------------------------------------------------------------------

        case Widget_Ids::Simulation_Actions_ListStore:
            if (not List_Of_Col_Texts.empty())
            THEN
                row[Simulation_Actions_ListStore_Cols.Time_Col] = List_Of_Col_Texts.front();
                List_Of_Col_Texts.pop_front();
            ENDIF
            if (not List_Of_Col_Texts.empty())
            THEN
                row[Simulation_Actions_ListStore_Cols.Action_Col] = List_Of_Col_Texts.front();
                List_Of_Col_Texts.pop_front();
            ENDIF
//            if (not List_Of_Col_Texts.empty())
//            THEN
//                row[Simulation_Actions_ListStore_Cols.Number_Col] = List_Of_Col_Texts.front();
//                List_Of_Col_Texts.pop_front();
//            ENDIF
            if (not List_Of_Col_Texts.empty())
            THEN
                row[Simulation_Actions_ListStore_Cols.Parameters_Col] = List_Of_Col_Texts.front();
                List_Of_Col_Texts.pop_front();
            ENDIF
            if (not List_Of_Col_Texts.empty())
            THEN
                row[Simulation_Actions_ListStore_Cols.Repeat_Col] = List_Of_Col_Texts.front();
                List_Of_Col_Texts.pop_front();
            ENDIF

            Assert(List_Of_Col_Texts.empty(), "GUI_Presentation::Add_Row - Too many texts for Simulation_Actions_ListStore");

            break;
//----------------------------------------------------------------------------------------------------------------

        case Widget_Ids::SysMon_ListStore:
            if (not List_Of_Col_Texts.empty())
            THEN
                row[SysMon_ListStore_Cols.Class_Col] = List_Of_Col_Texts.front();
                List_Of_Col_Texts.pop_front();
            ENDIF
            if (not List_Of_Col_Texts.empty())
            THEN
                row[SysMon_ListStore_Cols.Object_Count_Col] = List_Of_Col_Texts.front();
                List_Of_Col_Texts.pop_front();
            ENDIF
            if (not List_Of_Col_Texts.empty())
            THEN
                row[SysMon_ListStore_Cols.Creations_Col] = List_Of_Col_Texts.front();
                List_Of_Col_Texts.pop_front();
            ENDIF
            if (not List_Of_Col_Texts.empty())
            THEN
                row[SysMon_ListStore_Cols.Deletions_Col] = List_Of_Col_Texts.front();
                List_Of_Col_Texts.pop_front();
            ENDIF

            Assert(List_Of_Col_Texts.empty(), "GUI_Presentation::Add_Row - Too many texts for SysMon_ListStore");

            break;
//----------------------------------------------------------------------------------------------------------------
        default:
            Assert(true == false, "GUI_Presentation::Add_Row - Invalid ListStore_Id");
            break;
    }     
}
//----------------------------------------------------------------------------------------------------------------

void GUI_Presentation::Add_Text(Widget_Ids::TextBuffer_Id Id, string&  Text)

//   Add the specified text to the TextBuffer

{
    List_Of_TextBuffers[Id]->insert(List_Of_TextBuffers[Id]->end(),Text);
}
//----------------------------------------------------------------------------------------------------------------

void GUI_Presentation::Clear(Widget_Ids::TextBuffer_Id Id)

//  Clear the specified text buffer

{
    TRACE("GUI_Presentation::Clear");
    List_Of_TextBuffers[Id]->erase(List_Of_TextBuffers[Id]->begin(), List_Of_TextBuffers[Id]->end());
}
//----------------------------------------------------------------------------------------------------------------
void GUI_Presentation::Bind_Drawing_Area_To_API(Widget_Ids::Drawing_Area_Id  Id, 
                                                Drawing_Area*  Drawing_Area_Ptr)

//  Bind the specified Drawing Area to the Id used by the GUI Presentation API

{
    Assert(Id >= 0, "GUI_Presentation::Bind_Drawing_Area_To_API - Negative Drawing_Area_Id");
    Assert(Id < Widget_Ids::Num_Draw_Areas, 
           "GUI_Presentation::Bind_Drawing_Area_To_API - Out of Range Drawing_Area_Id ");

    if (List_Of_Drawing_Areas.size() < Widget_Ids::Num_Draw_Areas)
    THEN
        List_Of_Drawing_Areas.resize(Widget_Ids::Num_Draw_Areas);
    ENDIF

    List_Of_Drawing_Areas[Id] = Drawing_Area_Ptr;
}
//----------------------------------------------------------------------------------------------------------------
void GUI_Presentation::Bind_ListStore_To_API(Widget_Ids::ListStore_Id  Id, 
                                      Glib::RefPtr<Gtk::ListStore> ListStore_RefPtr)

//  Bind the specified ListStore to the ListStore Id used by the GUI Presentation API

{
    Assert(Id >= 0, "GUI_Presentation::Bind_ListStore_To_API - Negative ListStore Id");
    Assert(Id < Widget_Ids::Num_ListStores, 
           "GUI_Presentation::Bind_ListStore_To_API - Out of Range ListStore Id");

    if (List_Of_ListStores.size() < Widget_Ids::Num_ListStores)
    THEN
        List_Of_ListStores.resize(Widget_Ids::Num_ListStores);
    ENDIF

    List_Of_ListStores[Id] = ListStore_RefPtr;
}
//----------------------------------------------------------------------------------------------------------------

void GUI_Presentation::Bind_ListView_To_API(Widget_Ids::ListView_Id  Id, 
                                            Gtk::TreeView* ListView_Ptr)

//  Bind the specified ListView to the ListView Id used by the GUI Presentation API

{
    Assert(Id >= 0, "GUI_Presentation::Bind_ListView_To_API - Negative ListView Id");
    Assert(Id < Widget_Ids::Num_ListViews, 
           "GUI_Presentation::Bind_ListView_To_API - Out of Range ListView Id");

    if (List_Of_ListViews.size() < Widget_Ids::Num_ListViews)
    THEN
        List_Of_ListViews.resize(Widget_Ids::Num_ListViews);
    ENDIF

    List_Of_ListViews[Id] = ListView_Ptr;
}
//----------------------------------------------------------------------------------------------------------------
void GUI_Presentation::Bind_TextBuffer_To_API(Widget_Ids::TextBuffer_Id  Id, 
                                              Glib::RefPtr<Gtk::TextBuffer> TextBuffer_RefPtr)

//  Bind the specified TextBuffer to the TextBuffer Id used by the GUI Presentation API

{
    Assert(Id >= 0, "GUI_Presentation::Bind_TextBuffer_To_API - Negative TextBuffer Id");
    Assert(Id < Widget_Ids::Num_TextBuffers, 
           "GUI_Presentation::Bind_TextBuffer_To_API - Out of Range TextBuffer Id");

    if (List_Of_TextBuffers.size() < Widget_Ids::Num_TextBuffers)
    THEN
        List_Of_TextBuffers.resize(Widget_Ids::Num_TextBuffers);
    ENDIF

    List_Of_TextBuffers[Id] = TextBuffer_RefPtr;
}
//----------------------------------------------------------------------------------------------------------------

bool GUI_Presentation::Choose_File(string& PathName)

{
    TRACE("GUI_Presentation::Choose_File");
    Gtk::FileChooserDialog dialog("Please choose a file",
          Gtk::FILE_CHOOSER_ACTION_OPEN);
//  dialog.set_transient_for(*this);

    //Add response buttons the the dialog:
    dialog.add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
    dialog.add_button("Select", Gtk::RESPONSE_OK);

    int Result = dialog.run();

    if (Result == Gtk::RESPONSE_OK)
    THEN
        PathName = dialog.get_filename();
        return true;
    ELSEIF (Result == Gtk::RESPONSE_CANCEL)
    THEN
        return false;
    ELSE
        Assert(false, "GUI_Presentation::Choose_File - Invalid response code");
        return false;
    ENDIF
}
//----------------------------------------------------------------------------------------------------------------

void GUI_Presentation::Clear(Widget_Ids::Drawing_Area_Id  Drawing_Area_Spec)

//  Clear the specified Drawing Area

{
    List_Of_Drawing_Areas[Drawing_Area_Spec]->Clear();
}
//----------------------------------------------------------------------------------------------------------------

void GUI_Presentation::Clear(Widget_Ids::ListStore_Id Id)

//  Clear the specified ListStore

{
    List_Of_ListStores[Id]->clear();
}
//----------------------------------------------------------------------------------------------------------------

void GUI_Presentation::Draw_Line(Widget_Ids::Drawing_Area_Id  Drawing_Area_Spec,
                                 int X1, int Y1, int X2, int Y2)

//  Draw a line on the specified Drawing Area

{
    List_Of_Drawing_Areas[Drawing_Area_Spec]->Draw_Line(X1, Y1, X2, Y2);
}
//-----------------------------------------------------------------------------------------------------------------

void GUI_Presentation::Draw_Pixmap(Widget_Ids::Drawing_Area_Id   Drawing_Area_Spec)

//  Display the specified Drawing Area's Pixmap on the visible Window

{
    List_Of_Drawing_Areas[Drawing_Area_Spec]->Draw_Pixmap();
}
//-----------------------------------------------------------------------------------------------------------------

void GUI_Presentation::Draw_Rectangle(Widget_Ids::Drawing_Area_Id   Drawing_Area_Spec,
                                      bool Filled, int X, int  Y, int Width, int Height)

//  Draw a rectangle on the specified Drawing Area

{
    List_Of_Drawing_Areas[Drawing_Area_Spec]->Draw_Rectangle(Filled, X, Y, Width, Height);
}
//----------------------------------------------------------------------------------------------------------------


string GUI_Presentation::Get_Active_Page_Label()

//  Get the label for the active page 

{
    return SimSoup_Main_Notebook_Ptr->get_current()->get_tab_label_text();
}
//----------------------------------------------------------------------------------------------------------------

void GUI_Presentation::Get_Row_N_Col1(Widget_Ids::ListStore_Id Id, int RowNum, string& Row_Tx)

//  Get the first text for the Nth row in the specified ListStore.
//  First Row is row zero

{
    TRACE("GUI_Presentation::Get_Row_N");
    Assert(RowNum >= 0, "GUI_Presentation::Get_Row_N - Negative RowNum");

    Gtk::TreeModel::Children Rows = List_Of_ListStores[Id]->children();

    uint RowNum_Uns = static_cast<uint>  (RowNum);  // Avoids uint / int conversion warning
    Assert (RowNum_Uns < Rows.size(), "GUI_Presentation::Get_Row_N_Col1 - Row Number out of bounds");

    Gtk::TreeRow Row = Rows[RowNum];
    Row.get_value(0, Row_Tx);
}
//----------------------------------------------------------------------------------------------------------------

bool GUI_Presentation::Get_Selected_Row(Widget_Ids::ListView_Id Id, 
                                        list<string>& List_Of_Col_Texts, 
                                        void*& Data_Ptr)
     
//  Get the ListStore data for the selected row in the specified listView,
//  Also get the associated data

{
    bool Selected_Fl = false;
    switch (Id)
    {
        case Widget_Ids::MolTypes_ListView:
            BEGIN
                Gtk::TreeModel::iterator Iter = List_Of_ListViews[Id]->get_selection()->get_selected();
                Gtk::TreeModel::Row row = *Iter;
                if (Iter)  // Check whether row selected
                THEN
                    List_Of_Col_Texts.push_back(row[MolType_ListStore_Cols.MolType_Id_Col]);
                    List_Of_Col_Texts.push_back(row[MolType_ListStore_Cols.Mass_Col]);
                    List_Of_Col_Texts.push_back(row[MolType_ListStore_Cols.Potential_Energy_Col]);
                    Data_Ptr = row[MolType_ListStore_Cols.MolType_Ptr_Col];   
                    Selected_Fl = true;
                ELSE
                    Data_Ptr = 0;
                    Selected_Fl = false;
                ENDIF
            END
            break;

        case Widget_Ids::R_MolTypes_ListView:
            BEGIN
                Gtk::TreeModel::iterator Iter = List_Of_ListViews[Id]->get_selection()->get_selected();
                Gtk::TreeModel::Row row = *Iter;
                if (Iter)  // Check whether row selected
                THEN
                    List_Of_Col_Texts.push_back(row[R_MolType_ListStore_Cols.R_MolType_Id_Col]);
                    List_Of_Col_Texts.push_back(row[R_MolType_ListStore_Cols.Concentration_Col]);
                    Data_Ptr = row[R_MolType_ListStore_Cols.R_MolType_Ptr_Col];   
                    Selected_Fl = true;
                ELSE
                    Data_Ptr = 0;
                    Selected_Fl = false;
                ENDIF
            END
            break;

        case Widget_Ids::SimStats_Groups_ListView:
            BEGIN
                Gtk::TreeModel::iterator Iter = List_Of_ListViews[Id]->get_selection()->get_selected();
                Gtk::TreeModel::Row row = *Iter;
                if (Iter)  // Check whether row selected
                THEN
                    List_Of_Col_Texts.push_back(row[SimStats_ListStore_Groups_Cols.Group_Name_Col]);
                    List_Of_Col_Texts.push_back(row[SimStats_ListStore_Groups_Cols.Display_Flag_Col]);
                    Data_Ptr = row[SimStats_ListStore_Groups_Cols.Display_Group_Spec_Ptr_Col];               
                    Selected_Fl = true;
                ELSE
                    Data_Ptr = 0;
                    Selected_Fl = false;
                ENDIF
            END
            break;

        case Widget_Ids::SimStats_Series_ListView:
            BEGIN
                Gtk::TreeModel::iterator Iter = List_Of_ListViews[Id]->get_selection()->get_selected();
                Gtk::TreeModel::Row row = *Iter;
                if (Iter)  // Check whether row selected
                THEN
                    List_Of_Col_Texts.push_back(row[SimStats_ListStore_Series_Cols.Series_Name_Col]);
                    List_Of_Col_Texts.push_back(row[SimStats_ListStore_Series_Cols.Plot_Flag_Col]);
                    Data_Ptr = row[SimStats_ListStore_Series_Cols.Indexed_Series_Ptr_Col];               
                    Selected_Fl = true;
                ELSE
                    Data_Ptr = 0;
                    Selected_Fl = false;
                ENDIF
            END
            break;

       default:
            Assert(true == false, "GUI_Presentation::Get_Selected_Row - Invalid ListStore_Id");
            break;
    }   
    return Selected_Fl;
}
//-----------------------------------------------------------------------------------------------------------------

bool GUI_Presentation::Get_Selected_Row(Widget_Ids::ListView_Id Id, 
                                        list<string>& List_Of_Col_Texts)

//  Get the ListStore data for the selected row in the specified listView,

{
    TRACE("GUI_Presentation::Get_Selected_Row(Widget_Ids::ListView_Id Id, list<string>& List_Of_Col_Texts)");
    bool Selected_Fl = false;
    switch (Id)
    BEGIN
        case Widget_Ids::SimStats_SerList_ListView:
            BEGIN
                Gtk::TreeModel::iterator Iter = List_Of_ListViews[Id]->get_selection()->get_selected();
                Gtk::TreeModel::Row row = *Iter;
                if (Iter)  // Check whether row selected
                THEN
                    List_Of_Col_Texts.push_back(row[SimStats_ListStore_SerList_Cols.SerList_Name_Col]);
                    List_Of_Col_Texts.push_back(row[SimStats_ListStore_SerList_Cols.Display_Flag_Col]);                    
                    Selected_Fl = true;
                ELSE
                    Selected_Fl = false;
                ENDIF
            END
            break;

       default:
            Assert(true == false, "GUI_Presentation::Get_Selected_Row(Widget_Ids::ListView_Id Id, list<string>& List_Of_Col_Texts) - Invalid ListStore_Id");
            break;
    END
    return Selected_Fl;
}
//-----------------------------------------------------------------------------------------------------------------

void GUI_Presentation::Get_Size(Widget_Ids::Drawing_Area_Id   Drawing_Area_Spec, int& Width, int&  Height)

//  Get the size of the specified Drawing Area

{
    List_Of_Drawing_Areas[Drawing_Area_Spec]->Get_Size(Width, Height);
}
//----------------------------------------------------------------------------------------------------------------
        
void GUI_Presentation::Handle_File_Exit_Selection()

//  Handler for selection of the File->Exit menu item

{
//    MainForm::Get_Ptr()->File_Exit();
    User_Actions_API::MainForm_::File_Exit();
}
//----------------------------------------------------------------------------------------------------------------

void GUI_Presentation::Handle_File_Load_Scen_Selection()

//  Handler for selection of the File->Load Scenario menu item

{
    TRACE("GUI_Presentation::Handle_File_Load_Scen_Selection");
/*
   Gtk::FileChooserDialog dialog("Please choose a file",
          Gtk::FILE_CHOOSER_ACTION_OPEN);
//  dialog.set_transient_for(*this);

    //Add response buttons the the dialog:
    dialog.add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
    dialog.add_button("Select", Gtk::RESPONSE_OK);

    int result = dialog.run();



  //Handle the response:
  switch(result)
  {
    case(Gtk::RESPONSE_OK):
    {
      std::cout << "Open clicked." << std::endl;

      //Notice that this is a std::string, not a Glib::ustring.
      std::string filename = dialog.get_filename();
      std::cout << "File selected: " <<  filename << std::endl;
      break;
    }
    case(Gtk::RESPONSE_CANCEL):
    {
      std::cout << "Cancel clicked." << std::endl;
      break;
    }
    default:
    {
      std::cout << "Unexpected button clicked." << std::endl;
      break;
    }
  }


    string PathName = dialog.get_filename();*/

    if (not User_Actions_API::MainForm_::File_Load_Scenario())
    THEN
//      Could not load scenario
    ELSE
//      Scenario loaded. Disable the Load Scenario menu item and enable Close Scenario
        SimSoup_Main_UIManager_RefPtr->get_widget("/MenuBar/FileMenu/FileLoadScen")->set_sensitive(false);
        SimSoup_Main_UIManager_RefPtr->get_widget("/MenuBar/FileMenu/FileCloseScen")->set_sensitive(true);

//      And enable running of the simulation
        SimSoup_Main_UIManager_RefPtr->get_widget("/MenuBar/SimulationMenu/SimulationRun")->set_sensitive(true);
    ENDIF
}
//----------------------------------------------------------------------------------------------------------------

void GUI_Presentation::Handle_Main_Notebook_Switch_Page(GtkNotebookPage* page, guint page_num)

//  Handler for switching to a new notebook tab

{
    if (page_num == NoteBook_Tab_Ids.SysMon_Tab_Id)
    THEN
        User_Actions_API::Form_SysMon::Page_Select();
//    ELSEIF(page_num == NoteBook_Tab_Ids.SimControl_Tab_Id)
//    THEN
//        User_Actions_API::Form_SimCtrl::Page_Select();
//    ELSEIF(page_num == NoteBook_Tab_Ids.ChemReactr_Views_Tab_Id)
//    THEN
//        User_Actions_API::Form_ViewChem::Page_Select();
//    ELSEIF
//        User_Actions_API::Form_ViewReactor::Page_Select();
    ELSEIF (page_num == NoteBook_Tab_Ids.SimStats_Tab_Id)
    THEN
        User_Actions_API::Form_SimStats::Page_Select();      
    ENDIF
}
//----------------------------------------------------------------------------------------------------------------

void GUI_Presentation::Handle_File_Close_Scen_Selection()

//  Handler for selection of the File->Close Scenario menu item

{
    TRACE("GUI_Presentation::Handle_File_Close_Scen_Selection");
    User_Actions_API::MainForm_::File_Close_Scenario();

//  Scenario closed. Enable the Load Scenario menu item and disable Close Scenario
    SimSoup_Main_UIManager_RefPtr->get_widget("/MenuBar/FileMenu/FileLoadScen")->set_sensitive(true);
    SimSoup_Main_UIManager_RefPtr->get_widget("/MenuBar/FileMenu/FileCloseScen")->set_sensitive(false);

//  And disable running of the simulation
    SimSoup_Main_UIManager_RefPtr->get_widget("/MenuBar/SimulationMenu/SimulationRun")->set_sensitive(false);
}
//----------------------------------------------------------------------------------------------------------------

void GUI_Presentation::Handle_MolType_ListView_Row_Select()

//  Handler for selection of a row in the Molecule Types ListView on the Chemistry View

{
    list<string>  List_Of_Col_Texts;
    void*   Data_Ptr;
    Get_Selected_Row(Widget_Ids::MolTypes_ListView, 
                                        List_Of_Col_Texts, 
                                        Data_Ptr);
    if (Data_Ptr)  // Check whether row selected
    THEN
//        TForm__View_Chemistry::Get_Ptr()->Notify_MolType_ListView_Selection(Data_Ptr);
        User_Actions_API::Form_ViewChem::Notify_MolType_ListView_Selection(Data_Ptr);
    ENDIF
}
//----------------------------------------------------------------------------------------------------------------

void GUI_Presentation::Handle_R_MolType_ListView_Row_Select()

{
    list<string>  List_Of_Col_Texts;
    void*   Data_Ptr;
    Get_Selected_Row(Widget_Ids::R_MolTypes_ListView, 
                                        List_Of_Col_Texts, 
                                        Data_Ptr);
    if (Data_Ptr)  // Check whether row selected
    THEN
//        TForm__View_Reactor::Get_Ptr()->Notify_R_MolType_ListView_Selection(Data_Ptr);
        User_Actions_API::Form_ViewReactor::Notify_R_MolType_ListView_Selection(Data_Ptr);
    ENDIF
}
//----------------------------------------------------------------------------------------------------------------

void GUI_Presentation::Handle_SimStats_ListView_Groups_Row_DblClick(const Gtk::TreeModel::Path &, Gtk::TreeViewColumn*)

//  Handler for double click on the SimStats Groups ListView

{
//    Form_Simulation_Statistics::Get_Ptr()->ListView_Group_DblClick();
    User_Actions_API::Form_SimStats::ListView_Group_DblClick();
}
//----------------------------------------------------------------------------------------------------------------

void GUI_Presentation::Handle_SimStats_ListView_Series_Row_DblClick(const Gtk::TreeModel::Path &, Gtk::TreeViewColumn*)

//  Handler for double click on the SimStats Series ListView

{
    TRACE("GUI_Presentation::Handle_SimStats_ListView_Series_Row_DblClick");
    User_Actions_API::Form_SimStats::ListView_Series_DblClick();
}
//----------------------------------------------------------------------------------------------------------------

// void GUI_Presentation::Handle_SimStats_ListView_SerList_Row_DblClick(const Gtk::TreeModel::Path &, Gtk::TreeViewColumn*)
// 
// //  Handler for a double click on the Series List ListView
// 
// {
//     User_Actions_API::Form_SimStats::ListView_SerList_DblClick();
// }
//---------------------------------------------------------------------------------------------------------------

void GUI_Presentation::Handle_SimStats_ListView_SerList_Row_Select()

//  Handler for selection of a row in the Series List ListView on the Simulation Statistics Tab

{
     User_Actions_API::Form_SimStats::ListView_SerList_Select();
}
//----------------------------------------------------------------------------------------------------------------

void GUI_Presentation::Handle_Simulation_Run_Selection()

//  Handler for selection of the Simulation->Run menu item

{

    if (not User_Actions_API::MainForm_::Simulation_Go_Click())
    THEN
//      Could not start simulation
        return;  
    ELSE
//      Simulation started. Disable the Run Menu Item and enable the Stop Menu Item
        SimSoup_Main_UIManager_RefPtr->get_widget("/MenuBar/SimulationMenu/SimulationRun")->set_sensitive(false);
        SimSoup_Main_UIManager_RefPtr->get_widget("/MenuBar/SimulationMenu/SimulationStop")->set_sensitive(true);

//      Cannot close scenario while simulation running
        SimSoup_Main_UIManager_RefPtr->get_widget("/MenuBar/FileMenu/FileCloseScen")->set_sensitive(false);
    ENDIF
}
//----------------------------------------------------------------------------------------------------------------

void GUI_Presentation::Handle_Simulation_Stop_Selection()

//  Handler for selection of the Simulation->Stop menu item

{
    User_Actions_API::MainForm_::Simulation_Stop_Click();

//  Enable the Run Menu Item and disable the Stop Menu Item
    SimSoup_Main_UIManager_RefPtr->get_widget("/MenuBar/SimulationMenu/SimulationRun")->set_sensitive(true);
    SimSoup_Main_UIManager_RefPtr->get_widget("/MenuBar/SimulationMenu/SimulationStop")->set_sensitive(false);

//  Can close scenario while simulation stopped
    SimSoup_Main_UIManager_RefPtr->get_widget("/MenuBar/FileMenu/FileCloseScen")->set_sensitive(true);
}
//----------------------------------------------------------------------------------------------------------------
/*
void GUI_Presentation::Run_UI()

//  Start the main Gtk event loop

{
//    Gtk::Main::instance()->run();
    Gtk_Main_Instance_Ptr->run();
} */
//-----------------------------------------------------------------------------------------------------------------

bool  GUI_Presentation::Select_Row(Widget_Ids::ListView_Id Id, const string& Row_Tx)

//  Select the row in the specified ListView whose 
//  first column matches the specified text

{
    TRACE("GUI_Presentation::Select_Row");
    Gtk::TreeModel::Row Curr_Row; 
    for (uint ILoop = 0;
         ILoop < List_Of_ListStores[Widget_Ids::SimStats_SerList_ListStore]->children().size();
         ++ILoop)
    BEGIN
    
        Curr_Row = List_Of_ListStores[Widget_Ids::SimStats_SerList_ListStore]->children()[ILoop]; 
        string Curr_Row_Tx;
        Get_Row_N_Col1(Widget_Ids::SimStats_SerList_ListStore, ILoop, Curr_Row_Tx);

        if (Curr_Row_Tx == Row_Tx)
        THEN
            Glib::RefPtr<Gtk::TreeSelection> TreeSel =List_Of_ListViews[Widget_Ids::SimStats_SerList_ListView]->get_selection();
            TreeSel->select(Curr_Row); 
            return true;
        ENDIF
    END
    Assert(false, "GUI_Presentation::Select_Row - could not find row");
    return false; // Never gets here
}
//----------------------------------------------------------------------------------------------------------------

void GUI_Presentation::Set_Foreground_Colour(Widget_Ids::Drawing_Area_Id  Drawing_Area_Spec,
                                             Colour_Id  Colour_Spec)

//  Set the foreground colour for the specified Drawing Area to the specified SimSoup Colour,
//  using the mapping in the Colour_List

{
    List_Of_Drawing_Areas[Drawing_Area_Spec]->Set_Foreground_Colour(List_Of_Colours[Colour_Spec]);
} 
//-----------------------------------------------------------------------------------------------------------------

void GUI_Presentation::Set_Foreground_Colour(Widget_Ids::Drawing_Area_Id  Drawing_Area_Spec,
                                             int Red, int Green, int Blue)

//  Set the foreground colour for the specified Drawing Area to the specified RGB combination

{
    List_Of_Drawing_Areas[Drawing_Area_Spec]->Set_Foreground_Colour(Red, Green, Blue);
} 
//----------------------------------------------------------------------------------------------------------------

void GUI_Presentation::Set_Selection_Mode(Widget_Ids::ListView_Id Id,
                                          GUI::ListView_SelMode   Mode)

//  Set the selection mode for the specified ListView

{
    Glib::RefPtr<Gtk::TreeSelection> Sel_Pt = List_Of_ListViews[Id]->get_selection();

    if (Mode == GUI::Multiple) Sel_Pt->set_mode(Gtk::SELECTION_MULTIPLE);
    if (Mode == GUI::Single)   Sel_Pt->set_mode(Gtk::SELECTION_SINGLE);
    if (Mode == GUI::None)     Sel_Pt->set_mode(Gtk::SELECTION_NONE);
}
//----------------------------------------------------------------------------------------------------------------

void GUI_Presentation::Setup_Chem_And_Reactor_Tab()

//  Setup the Chemistry and Reactor View Tab

{

//  Divide the tab into Chemistry and Reactor View areas

    CREATE_OBJECT(Gtk::HPaned, Chem_Reactor_Paned_Ptr = new Gtk::HPaned);

//  Widgets for the Chemistry View

    CREATE_OBJECT(Gtk::VPaned, Chem_View_Paned_Ptr = new Gtk::VPaned);
    CREATE_OBJECT(Gtk::HPaned, Chem_ListViews_Paned_Ptr = new Gtk::HPaned);

    CREATE_OBJECT(Drawing_Area, Chem_Drawing_Area_Ptr = new Drawing_Area);
    CREATE_OBJECT(Gtk::TreeView, MolType_ListView_Ptr = new Gtk::TreeView);
    CREATE_OBJECT(Gtk::ScrolledWindow, MolType_ListView_ScrolledWindow_Ptr = new  Gtk::ScrolledWindow);
    CREATE_OBJECT(Gtk::TreeView, IntType_ListView_Ptr = new Gtk::TreeView);
    CREATE_OBJECT(Gtk::ScrolledWindow, IntType_ListView_ScrolledWindow_Ptr = new  Gtk::ScrolledWindow);

//  Widgets for the Reactor View

    CREATE_OBJECT(Gtk::VPaned, Reactor_View_Paned_Ptr = new Gtk::VPaned);
    CREATE_OBJECT(Gtk::HPaned, Reactor_ListViews_Paned_Ptr = new Gtk::HPaned);

    CREATE_OBJECT(Drawing_Area, Reactor_Drawing_Area_Ptr = new Drawing_Area);
    CREATE_OBJECT(Gtk::TreeView, R_MolType_ListView_Ptr = new Gtk::TreeView);
    CREATE_OBJECT(Gtk::ScrolledWindow, R_MolType_ListView_ScrolledWindow_Ptr = new  Gtk::ScrolledWindow);
    CREATE_OBJECT(Gtk::TreeView, R_IntType_ListView_Ptr = new Gtk::TreeView);
    CREATE_OBJECT(Gtk::ScrolledWindow, R_IntType_ListView_ScrolledWindow_Ptr = new  Gtk::ScrolledWindow);

//----------------------------------------------------------------------------------------------------------------

//  Place the widgets in their locations on the tab and set display properties

    Chem_Reactor_Paned_Ptr->pack1(*Chem_View_Paned_Ptr, Gtk::SHRINK);
    Chem_Reactor_Paned_Ptr->pack2(*Reactor_View_Paned_Ptr, Gtk::SHRINK);

//  Chemistry View

    MolType_ListView_ScrolledWindow_Ptr->add(*MolType_ListView_Ptr);
    MolType_ListView_ScrolledWindow_Ptr->set_policy(Gtk::POLICY_AUTOMATIC, Gtk::POLICY_AUTOMATIC);
    IntType_ListView_ScrolledWindow_Ptr->add(*IntType_ListView_Ptr);
    IntType_ListView_ScrolledWindow_Ptr->set_policy(Gtk::POLICY_AUTOMATIC, Gtk::POLICY_AUTOMATIC);

    Chem_View_Paned_Ptr->pack1(*Chem_ListViews_Paned_Ptr, Gtk::SHRINK); 
    Chem_View_Paned_Ptr->pack2(*Chem_Drawing_Area_Ptr, Gtk::SHRINK);

    Chem_ListViews_Paned_Ptr->pack1(*MolType_ListView_ScrolledWindow_Ptr);
    Chem_ListViews_Paned_Ptr->pack2(*IntType_ListView_ScrolledWindow_Ptr);

//  Set the position for the divider. Works best with Gtk::SHRINK

    int Width;
    int Height;
    SimSoup_Main_Window_Ptr->get_size(Width, Height);
    Chem_View_Paned_Ptr->set_position(Height/4);
//----------------------------------------------------------------------------------------------------------------

//  Setup the Molecule Type ListView and its ListStore

    MolType_ListView_Ptr->append_column("Molecule Type", MolType_ListStore_Cols.MolType_Id_Col); 
    MolType_ListView_Ptr->append_column("Mass", MolType_ListStore_Cols.Mass_Col);
    MolType_ListView_Ptr->append_column("Potential Energy", MolType_ListStore_Cols.Potential_Energy_Col);
    for(int ILoop = 0; ; ILoop++)
    {
        Gtk::TreeView::Column* Col_Ptr = MolType_ListView_Ptr->get_column(ILoop);
        if (Col_Ptr == 0) break;
        Col_Ptr->set_resizable();
    }

    MolType_ListStore_RefPtr = Gtk::ListStore::create(MolType_ListStore_Cols);
    MolType_ListView_Ptr->set_model(MolType_ListStore_RefPtr); 

    Bind_ListStore_To_API(Widget_Ids::MolTypes_ListStore, 
                          MolType_ListStore_RefPtr);

    Bind_ListView_To_API(Widget_Ids::MolTypes_ListView, 
                         MolType_ListView_Ptr);

//  Setup handler for selection of row in the MolType ListView

    Glib::RefPtr<Gtk::TreeSelection> MolType_ListView_TreeSelection_Ref_Ptr =
        MolType_ListView_Ptr->get_selection();

    MolType_ListView_TreeSelection_Ref_Ptr->signal_changed().connect(
        sigc::mem_fun(*this, &GUI_Presentation::Handle_MolType_ListView_Row_Select));
//----------------------------------------------------------------------------------------------------------------

//  Setup the Interaction Type ListView and its ListStore

    IntType_ListView_Ptr->append_column("Interaction Type", IntType_ListStore_Cols.IntType_Id_Col); 
    IntType_ListView_Ptr->append_column("Category", IntType_ListStore_Cols.Category_Col); 
    IntType_ListView_Ptr->append_column("Heat", IntType_ListStore_Cols.Heat_Col); 
    IntType_ListView_Ptr->append_column("Fwd. Actn. Energy", IntType_ListStore_Cols.Fwd_Actn_Energy_Col); 
    IntType_ListView_Ptr->append_column("Rev. Actn. Energy", IntType_ListStore_Cols.Rev_Actn_Energy_Col); 
    IntType_ListView_Ptr->append_column("Actv. Cmplx. Energy", IntType_ListStore_Cols.Actv_Cmplx_Energy_Col); 
    IntType_ListView_Ptr->append_column("Fwd. Rate. Const.", IntType_ListStore_Cols.Fwd_Rate_Const_Col); 
    IntType_ListView_Ptr->append_column("Rev. Rate. Const.", IntType_ListStore_Cols.Rev_Rate_Const_Col); 
    IntType_ListView_Ptr->append_column("Reactant 1", IntType_ListStore_Cols.Reactant1_Col); 
    IntType_ListView_Ptr->append_column("Reactant 2", IntType_ListStore_Cols.Reactant2_Col); 
    IntType_ListView_Ptr->append_column("Product 1", IntType_ListStore_Cols.Product1_Col); 
    IntType_ListView_Ptr->append_column("Product 2", IntType_ListStore_Cols.Product2_Col); 

    for(int ILoop = 0; ; ILoop++)
    BEGIN
        Gtk::TreeView::Column* Col_Ptr = IntType_ListView_Ptr->get_column(ILoop);
        if (Col_Ptr == 0) break;
        Col_Ptr->set_resizable();
    END

    IntType_ListStore_RefPtr = Gtk::ListStore::create(IntType_ListStore_Cols);
    IntType_ListView_Ptr->set_model(IntType_ListStore_RefPtr); 

    Bind_ListStore_To_API(Widget_Ids::IntTypes_ListStore, 
                          IntType_ListStore_RefPtr);

//----------------------------------------------------------------------------------------------------------------

//  Reactor View

    R_MolType_ListView_ScrolledWindow_Ptr->add(*R_MolType_ListView_Ptr);
    R_MolType_ListView_ScrolledWindow_Ptr->set_policy(Gtk::POLICY_AUTOMATIC, Gtk::POLICY_AUTOMATIC);

    R_IntType_ListView_ScrolledWindow_Ptr->add(*R_IntType_ListView_Ptr);
    R_IntType_ListView_ScrolledWindow_Ptr->set_policy(Gtk::POLICY_AUTOMATIC, Gtk::POLICY_AUTOMATIC);

    Reactor_View_Paned_Ptr->pack1(*Reactor_ListViews_Paned_Ptr, Gtk::SHRINK);
    Reactor_View_Paned_Ptr->pack2(*Reactor_Drawing_Area_Ptr, Gtk::SHRINK);

    Reactor_ListViews_Paned_Ptr->pack1(*R_MolType_ListView_ScrolledWindow_Ptr);
    Reactor_ListViews_Paned_Ptr->pack2(*R_IntType_ListView_ScrolledWindow_Ptr);


    Reactor_View_Paned_Ptr->set_position(Height/4);
//----------------------------------------------------------------------------------------------------------------

//  Setup the Realised Molecule Type ListView and its ListStore

    R_MolType_ListView_Ptr->append_column("Realised Mol. Type", R_MolType_ListStore_Cols.R_MolType_Id_Col); 
    R_MolType_ListView_Ptr->append_column("Concentration", R_MolType_ListStore_Cols.Concentration_Col);
    for(int ILoop = 0; ; ILoop++)
    BEGIN
        Gtk::TreeView::Column* Col_Ptr = R_MolType_ListView_Ptr->get_column(ILoop);
        if (Col_Ptr == 0) break;
        Col_Ptr->set_resizable();
    END

    R_MolType_ListStore_RefPtr = Gtk::ListStore::create(R_MolType_ListStore_Cols);
    R_MolType_ListView_Ptr->set_model(R_MolType_ListStore_RefPtr); 

    Bind_ListStore_To_API(Widget_Ids::R_MolType_ListStore, 
                          R_MolType_ListStore_RefPtr);

    Bind_ListView_To_API(Widget_Ids::R_MolTypes_ListView, 
                         R_MolType_ListView_Ptr);

//  Setup handler for selection of row in the Realised MolType ListView

    Glib::RefPtr<Gtk::TreeSelection> R_MolType_ListView_TreeSelection_Ref_Ptr =
        R_MolType_ListView_Ptr->get_selection();

    R_MolType_ListView_TreeSelection_Ref_Ptr->signal_changed().connect(
        sigc::mem_fun(*this, &GUI_Presentation::Handle_R_MolType_ListView_Row_Select));
//----------------------------------------------------------------------------------------------------------------

//  Setup the Realisable Interaction Type ListView and its ListStore

    R_IntType_ListView_Ptr->append_column("Realisable Int. Type", R_IntType_ListStore_Cols.R_IntType_Id_Col); 
    R_IntType_ListView_Ptr->append_column("Category", R_IntType_ListStore_Cols.Category_Col);
    R_IntType_ListView_Ptr->append_column("Rate", R_IntType_ListStore_Cols.Rate_Col);
    R_IntType_ListView_Ptr->append_column("Reactant 1", R_IntType_ListStore_Cols.Reactant1_Col);
    R_IntType_ListView_Ptr->append_column("Reactant 2", R_IntType_ListStore_Cols.Reactant2_Col);
    R_IntType_ListView_Ptr->append_column("Product 1", R_IntType_ListStore_Cols.Product1_Col);
    R_IntType_ListView_Ptr->append_column("Product 2", R_IntType_ListStore_Cols.Product2_Col);

    for(int ILoop = 0; ; ILoop++)
    BEGIN
        Gtk::TreeView::Column* Col_Ptr = R_MolType_ListView_Ptr->get_column(ILoop);
        if (Col_Ptr == 0) break;
        Col_Ptr->set_resizable();
    END

    R_IntType_ListStore_RefPtr = Gtk::ListStore::create(R_IntType_ListStore_Cols);
    R_IntType_ListView_Ptr->set_model(R_IntType_ListStore_RefPtr); 

    Bind_ListStore_To_API(Widget_Ids::R_IntType_ListStore, 
                          R_IntType_ListStore_RefPtr);

//----------------------------------------------------------------------------------------------------------------

//  Bind Chemistry Drawing Area

    Bind_Drawing_Area_To_API(Widget_Ids::Chemistry_Draw_Area, Chem_Drawing_Area_Ptr);

//  Bind Reactor Drawing Area

    Bind_Drawing_Area_To_API(Widget_Ids::Reactor_Draw_Area, Reactor_Drawing_Area_Ptr);

//----------------------------------------------------------------------------------------------------------------

//  Setup the Chemistry and Reactor View Tab in the Notebook. Record the Tab Id

    NoteBook_Tab_Ids.ChemReactr_Views_Tab_Id = 
    SimSoup_Main_Notebook_Ptr->append_page(*Chem_Reactor_Paned_Ptr, "Chemistry and Reactor Views");
}
//-----------------------------------------------------------------------------------------------------------------

void GUI_Presentation::Setup_Help_Tab()

//  Setup the Help Tab

{
    CREATE_OBJECT(Gtk::Notebook, Help_Notebook_Ptr = new Gtk::Notebook);
    SimSoup_Main_Notebook_Ptr->append_page(*Help_Notebook_Ptr, "Help");

//-----------------Quick Start -----------------------------------------------------------------------------------

//  Setup the Quick Start TextView

    CREATE_OBJECT(Gtk::TextView, Help_Quick_Start_TextView_Ptr = new Gtk::TextView);
    Help_Quick_Start_TextView_Ptr->set_wrap_mode(Gtk::WRAP_WORD);
    Help_Quick_Start_TextView_Ptr->set_editable(false);
    Help_Quick_Start_TextView_Ptr->set_cursor_visible(false);

//  Setup the buffer to hold the Help Quick Start text, and associate it with the Identifier used by the API

    Help_Quick_Start_TextBuf_RefPtr = Gtk::TextBuffer::create();
    Bind_TextBuffer_To_API(Widget_Ids::Help_Quick_Start_TextBuffer, 
                           Help_Quick_Start_TextBuf_RefPtr);

//  Associate the TextView with the TextBuffer

    Help_Quick_Start_TextView_Ptr->set_buffer(Help_Quick_Start_TextBuf_RefPtr);

//  Put the Help Quick Start TextView in a Scolled Window

    CREATE_OBJECT(Gtk::ScrolledWindow, Help_Quick_Start_ScrolledWindow_Ptr  = new  Gtk::ScrolledWindow);
    Help_Quick_Start_ScrolledWindow_Ptr->add(*Help_Quick_Start_TextView_Ptr);

    Help_Notebook_Ptr->append_page(*Help_Quick_Start_ScrolledWindow_Ptr, "Quick Start");

//--------------- Help About -------------------------------------------------------------------------------------

//  Setup the Help About TextView

    CREATE_OBJECT(Gtk::TextView, Help_About_TextView_Ptr = new Gtk::TextView);
    Help_About_TextView_Ptr->set_wrap_mode(Gtk::WRAP_WORD);
    Help_About_TextView_Ptr->set_editable(false);
    Help_About_TextView_Ptr->set_cursor_visible(false);

//  Setup the buffer to hold the Help About text, and associate it with the Identifier used by the API

    Help_About_TextBuf_RefPtr = Gtk::TextBuffer::create();
    Bind_TextBuffer_To_API(Widget_Ids::Help_About_TextBuffer, 
                           Help_About_TextBuf_RefPtr);

//  Associate the TextView with the TextBuffer

    Help_About_TextView_Ptr->set_buffer(Help_About_TextBuf_RefPtr);

//  Put the Help About TextView in a Scolled Window

    CREATE_OBJECT(Gtk::ScrolledWindow, Help_About_ScrolledWindow_Ptr  = new  Gtk::ScrolledWindow);
    Help_About_ScrolledWindow_Ptr->add(*Help_About_TextView_Ptr);

    Help_Notebook_Ptr->append_page(*Help_About_ScrolledWindow_Ptr, "About");

//----------------- Licence --------------------------------------------------------------------------------------

//  Setup the Licence TextView

    CREATE_OBJECT(Gtk::TextView, Help_Licence_TextView_Ptr = new Gtk::TextView);
    Help_Licence_TextView_Ptr->set_wrap_mode(Gtk::WRAP_WORD);
    Help_Licence_TextView_Ptr->set_editable(false);
    Help_Licence_TextView_Ptr->set_cursor_visible(false);

//  Setup the buffer to hold the Help Licence text, and associate it with the Identifier used by the API

    Help_Licence_TextBuf_RefPtr = Gtk::TextBuffer::create();
    Bind_TextBuffer_To_API(Widget_Ids::Help_Licence_TextBuffer, 
                           Help_Licence_TextBuf_RefPtr);

//  Associate the TextView with the TextBuffer

    Help_Licence_TextView_Ptr->set_buffer(Help_Licence_TextBuf_RefPtr);

//  Put the Help Licence TextView in a Scolled Window

    CREATE_OBJECT(Gtk::ScrolledWindow, Help_Licence_ScrolledWindow_Ptr  = new  Gtk::ScrolledWindow);
    Help_Licence_ScrolledWindow_Ptr->add(*Help_Licence_TextView_Ptr);

    NoteBook_Tab_Ids.Help_Tab_Id =
    Help_Notebook_Ptr->append_page(*Help_Licence_ScrolledWindow_Ptr, "Licence");

    User_Actions_API::Form_Help_::Show_Help_Text();  // Only need to do once as does not change
}
//----------------------------------------------------------------------------------------------------------------

void GUI_Presentation::Setup_Main_Menu()

//  Setup the Main Menu

{
    TRACE("GUI_Presentation::Setup_Main_Menu");
    SimSoup_Main_UIManager_RefPtr = Gtk::UIManager::create();
    SimSoup_Main_ActionGroup_RefPtr = Gtk::ActionGroup::create();

//  File menu

    SimSoup_Main_ActionGroup_RefPtr->add(Gtk::Action::create("FileMenu", "File"));

    SimSoup_Main_ActionGroup_RefPtr->add(Gtk::Action::create("FileLoadScen", 
                                         "Open Scenario"), 
                                         sigc::mem_fun(*this, &GUI_Presentation::Handle_File_Load_Scen_Selection));

    SimSoup_Main_ActionGroup_RefPtr->add(Gtk::Action::create("FileCloseScen", 
                                         "Close Scenario"), 
                                         sigc::mem_fun(*this, &GUI_Presentation::Handle_File_Close_Scen_Selection));


    SimSoup_Main_ActionGroup_RefPtr->add(Gtk::Action::create("FileExit", 
                                         Gtk::Stock::QUIT), 
                                         sigc::mem_fun(*this, &GUI_Presentation::Handle_File_Exit_Selection));

//  Simulation Menu

    SimSoup_Main_ActionGroup_RefPtr->add(Gtk::Action::create("SimulationMenu", "Simulation"));
    SimSoup_Main_ActionGroup_RefPtr->add(Gtk::Action::create("SimulationRun", 
                                         "Run Simulation"),
                                         sigc::mem_fun(*this, &GUI_Presentation::Handle_Simulation_Run_Selection));
    SimSoup_Main_ActionGroup_RefPtr->add(Gtk::Action::create("SimulationStop", 
                                         "Stop Simulation"),
                                         sigc::mem_fun(*this, &GUI_Presentation::Handle_Simulation_Stop_Selection));

    SimSoup_Main_UIManager_RefPtr->insert_action_group(SimSoup_Main_ActionGroup_RefPtr);

//  Layout the actions in a menubar

    Glib::ustring ui_info = 
        "<ui>"
        "  <menubar name='MenuBar'>"
        "    <menu action='FileMenu'>"
        "      <menuitem action='FileLoadScen'/>"
        "      <menuitem action='FileCloseScen'/>"
        "      <menuitem action='FileExit'/>"
        "    </menu>"
        "    <menu action='SimulationMenu'>"
        "      <menuitem action='SimulationRun'/>"
        "      <menuitem action='SimulationStop'/>"
        "    </menu>"
        "  </menubar>"
        "</ui>";
    SimSoup_Main_UIManager_RefPtr->add_ui_from_string(ui_info);

//  Close scenario item is initially disabled
    SimSoup_Main_UIManager_RefPtr->get_widget("/MenuBar/FileMenu/FileCloseScen")->set_sensitive(false);

//  Stop and Run Simulation are initially disabled
    SimSoup_Main_UIManager_RefPtr->get_widget("/MenuBar/SimulationMenu/SimulationStop")->set_sensitive(false);
    SimSoup_Main_UIManager_RefPtr->get_widget("/MenuBar/SimulationMenu/SimulationRun")->set_sensitive(false);

    Gtk::Widget* Menubar_Ptr = SimSoup_Main_UIManager_RefPtr->get_widget("/MenuBar");
    if(Menubar_Ptr)
    THEN
        SimSoup_Main_VBox_Ptr->pack_start(*Menubar_Ptr, Gtk::PACK_SHRINK);
    ELSE
        Assert(true == false, "GUI_Presentation::Start_UI() - Cannot create menu bar");
    ENDIF
}
//----------------------------------------------------------------------------------------------------------------

void GUI_Presentation::Setup_Simulation_Control_Tab()

//  Setup the Simulation Control Tab

{
    CREATE_OBJECT(Gtk::HPaned, SimControl_Paned_Ptr = new Gtk::HPaned);

//  Create the Actions ListView

    CREATE_OBJECT(Gtk::ScrolledWindow, SimControl_Actions_ScrolledWindow_Ptr = new  Gtk::ScrolledWindow);
    CREATE_OBJECT(Gtk::TreeView, SimControl_Actions_ListView_Ptr = new Gtk::TreeView);
    SimControl_Actions_ScrolledWindow_Ptr->add(*SimControl_Actions_ListView_Ptr);
//----------------------------------------------------------------------------------------------------------------

//  Setup the Log TextView

    CREATE_OBJECT(Gtk::TextView, SimControl_Log_TextView_Ptr = new Gtk::TextView);
    SimControl_Log_TextView_Ptr->set_wrap_mode(Gtk::WRAP_WORD);
    SimControl_Log_TextView_Ptr->set_editable(false);
    SimControl_Log_TextView_Ptr->set_cursor_visible(false);

//  Setup the buffer to hold the display text, and associate it with the Identifier used by the API

    SimControl_Log_TextBuf_RefPtr = Gtk::TextBuffer::create();
    Bind_TextBuffer_To_API(Widget_Ids::SimControl_Log_TextBuffer, 
                           SimControl_Log_TextBuf_RefPtr);

//  Associate the TextView with the TextBuffer

    SimControl_Log_TextView_Ptr->set_buffer(SimControl_Log_TextBuf_RefPtr);

//  Put the Log TextView in a Scolled Window

    CREATE_OBJECT(Gtk::ScrolledWindow, SimControl_Log_ScrolledWindow_Ptr  = new  Gtk::ScrolledWindow);
    SimControl_Log_ScrolledWindow_Ptr->add(*SimControl_Log_TextView_Ptr);

//----------------------------------------------------------------------------------------------------------------

//  Place the widgets on the Paned view

    SimControl_Paned_Ptr->pack1(*SimControl_Actions_ScrolledWindow_Ptr, Gtk::EXPAND);
    SimControl_Paned_Ptr->pack2(*SimControl_Log_ScrolledWindow_Ptr, Gtk::EXPAND);

//  Setup the Actions ListView and the ListStore

    SimControl_Actions_ListView_Ptr->append_column("Time", Simulation_Actions_ListStore_Cols.Time_Col); 
    SimControl_Actions_ListView_Ptr->append_column("Action", Simulation_Actions_ListStore_Cols.Action_Col); 
//    SimControl_Actions_ListView_Ptr->append_column("Number", Simulation_Actions_ListStore_Cols.Number_Col); 
    SimControl_Actions_ListView_Ptr->append_column("Parameters", Simulation_Actions_ListStore_Cols.Parameters_Col); 
    SimControl_Actions_ListView_Ptr->append_column("Repeat Interval", Simulation_Actions_ListStore_Cols.Repeat_Col); 

    for(int ILoop = 0; ; ILoop++)
    {
        Gtk::TreeView::Column* Col_Ptr = SimControl_Actions_ListView_Ptr->get_column(ILoop);
        if (Col_Ptr == 0) break;
        Col_Ptr->set_resizable();
    }

    Simulation_Actions_ListStore_RefPtr = Gtk::ListStore::create(Simulation_Actions_ListStore_Cols);
    SimControl_Actions_ListView_Ptr->set_model(Simulation_Actions_ListStore_RefPtr);

    Bind_ListStore_To_API(Widget_Ids::Simulation_Actions_ListStore, 
                          Simulation_Actions_ListStore_RefPtr);

//  Setup the Simulation Control Tab in the Notebook
//  Record the Tab Identifier
    NoteBook_Tab_Ids.SimControl_Tab_Id = 
    SimSoup_Main_Notebook_Ptr->append_page(*SimControl_Paned_Ptr, "Scenario");
}
//----------------------------------------------------------------------------------------------------------------

void  GUI_Presentation::Setup_SimStats_Tab()

//  Setup the Simulation Statistics Tab

{
//  First create screen widgets

    CREATE_OBJECT(Gtk::HPaned, SimStats_Main_Panes_Ptr = new Gtk::HPaned);
    CREATE_OBJECT(Drawing_Area, SimStats_Drawing_Area_Ptr = new Drawing_Area);
    CREATE_OBJECT(Gtk::VPaned, SimStats_ListV_Top_Panes_Pt = new Gtk::VPaned);
    CREATE_OBJECT(Gtk::VPaned, SimStats_ListV_Bot_Panes_Pt = new Gtk::VPaned);
    CREATE_OBJECT(Gtk::TreeView, SimStats_ListView_Groups_Ptr = new Gtk::TreeView);
    CREATE_OBJECT(Gtk::ScrolledWindow, SimStats_ListView_Groups_ScrolledWindow_Ptr = new  Gtk::ScrolledWindow);
    CREATE_OBJECT(Gtk::TreeView, SimStats_ListView_SerList_Pt = new Gtk::TreeView);
    CREATE_OBJECT(Gtk::ScrolledWindow, SimStats_ListView_SerList_ScrolledWindow_Pt = new  Gtk::ScrolledWindow);
    CREATE_OBJECT(Gtk::ScrolledWindow, SimStats_ListView_Series_ScrolledWindow_Ptr = new  Gtk::ScrolledWindow);
    CREATE_OBJECT(Gtk::TreeView, SimStats_ListView_Series_Ptr = new Gtk::TreeView);

//  Place the widgets in their appropriate holders

    SimStats_Main_Panes_Ptr->pack1(*SimStats_ListV_Top_Panes_Pt, Gtk::EXPAND);
    SimStats_Main_Panes_Ptr->pack2(*SimStats_Drawing_Area_Ptr, Gtk::EXPAND);

    SimStats_ListView_Groups_ScrolledWindow_Ptr->add(*SimStats_ListView_Groups_Ptr);
    SimStats_ListView_SerList_ScrolledWindow_Pt->add(*SimStats_ListView_SerList_Pt);
    SimStats_ListView_Series_ScrolledWindow_Ptr->add(*SimStats_ListView_Series_Ptr);

//  Only show the scrollbars when they are necessary:
    SimStats_ListView_Groups_ScrolledWindow_Ptr->set_policy(Gtk::POLICY_AUTOMATIC, Gtk::POLICY_AUTOMATIC);
    SimStats_ListView_SerList_ScrolledWindow_Pt->set_policy(Gtk::POLICY_AUTOMATIC, Gtk::POLICY_AUTOMATIC);
    SimStats_ListView_Series_ScrolledWindow_Ptr->set_policy(Gtk::POLICY_AUTOMATIC, Gtk::POLICY_AUTOMATIC);

    SimStats_ListV_Top_Panes_Pt->pack1(*SimStats_ListView_Groups_ScrolledWindow_Ptr, Gtk::EXPAND);
    SimStats_ListV_Top_Panes_Pt->pack2(*SimStats_ListV_Bot_Panes_Pt , Gtk::EXPAND);

    SimStats_ListV_Bot_Panes_Pt->pack1(*SimStats_ListView_SerList_ScrolledWindow_Pt , Gtk::EXPAND);
    SimStats_ListV_Bot_Panes_Pt->pack2(*SimStats_ListView_Series_ScrolledWindow_Ptr, Gtk::EXPAND);

//    SimStats_ListView_Panes_Ptr->pack2(*SimStats_ListView_Series_ScrolledWindow_Ptr, Gtk::EXPAND);

//  Setup the Groups ListView and its ListStore
    SimStats_ListView_Groups_Ptr->append_column("Group", SimStats_ListStore_Groups_Cols.Group_Name_Col); //This number will be shown with the default numeric formatting.
    SimStats_ListView_Groups_Ptr->append_column("Display",SimStats_ListStore_Groups_Cols.Display_Flag_Col);
    for(int i = 0; ; i++)
    {
        Gtk::TreeView::Column* Col_Ptr = SimStats_ListView_Groups_Ptr->get_column(i);
        if (Col_Ptr == 0) break;
        Col_Ptr->set_resizable();
    }

    SimStats_ListStore_Groups_RefPtr = Gtk::ListStore::create(SimStats_ListStore_Groups_Cols);
    SimStats_ListView_Groups_Ptr->set_model(SimStats_ListStore_Groups_RefPtr);

    Bind_ListStore_To_API(Widget_Ids::SimStats_Groups_ListStore, 
                          SimStats_ListStore_Groups_RefPtr);

    Bind_ListView_To_API(Widget_Ids::SimStats_Groups_ListView, 
                         SimStats_ListView_Groups_Ptr);

//  Setup the Series List ListView and its ListStore
    SimStats_ListView_SerList_Pt->append_column("Series List", SimStats_ListStore_SerList_Cols.SerList_Name_Col); 
    SimStats_ListView_SerList_Pt->append_column("Display", SimStats_ListStore_SerList_Cols.Display_Flag_Col); 

    for(int i = 0; ; i++)
    {
        Gtk::TreeView::Column* Col_Ptr = SimStats_ListView_SerList_Pt->get_column(i);
        if (Col_Ptr == 0) break;
        Col_Ptr->set_resizable();
    }
    SimStats_ListStore_SerList_RefPtr = Gtk::ListStore::create(SimStats_ListStore_SerList_Cols);
    SimStats_ListView_SerList_Pt->set_model(SimStats_ListStore_SerList_RefPtr); 

    Bind_ListStore_To_API(Widget_Ids::SimStats_SerList_ListStore, 
                          SimStats_ListStore_SerList_RefPtr );

    Bind_ListView_To_API(Widget_Ids::SimStats_SerList_ListView, 
                         SimStats_ListView_SerList_Pt);

//  Setup the Series ListView and its ListStore
    SimStats_ListView_Series_Ptr->append_column("Series", SimStats_ListStore_Series_Cols.Series_Name_Col); 
    SimStats_ListView_Series_Ptr->append_column("Plot", SimStats_ListStore_Series_Cols.Plot_Flag_Col);
    for(int i = 0; ; i++)
    {
        Gtk::TreeView::Column* Col_Ptr = SimStats_ListView_Series_Ptr->get_column(i);
        if (Col_Ptr == 0) break;
        Col_Ptr->set_resizable();
    }
    SimStats_ListStore_Series_RefPtr = Gtk::ListStore::create(SimStats_ListStore_Series_Cols);
    SimStats_ListView_Series_Ptr->set_model(SimStats_ListStore_Series_RefPtr); 

    Bind_ListStore_To_API(Widget_Ids::SimStats_Series_ListStore, 
                          SimStats_ListStore_Series_RefPtr);

    Bind_ListView_To_API(Widget_Ids::SimStats_Series_ListView, 
                         SimStats_ListView_Series_Ptr);

//  Setup handlers for selection of rows in the ListViews

    Glib::RefPtr<Gtk::TreeSelection> SimStats_ListView_Series_TreeSelection_Ref_Ptr =
    SimStats_ListView_Series_Ptr->get_selection();

    SimStats_ListView_Series_Ptr->signal_row_activated().connect(
    sigc::mem_fun(*this, &GUI_Presentation::Handle_SimStats_ListView_Series_Row_DblClick));

    SimStats_ListView_Groups_Ptr->signal_row_activated().connect(
    sigc::mem_fun(*this, &GUI_Presentation::Handle_SimStats_ListView_Groups_Row_DblClick));

//    SimStats_ListView_SerList_Pt->signal_row_activated().connect(
//    sigc::mem_fun(*this, &GUI_Presentation::Handle_SimStats_ListView_SerList_Row_DblClick));

//  Setup handler for selection of row in the Series List ListView

    Glib::RefPtr<Gtk::TreeSelection> SerList_ListView_TreeSelection_Ref_Ptr =
        SimStats_ListView_SerList_Pt->get_selection();

    SerList_ListView_TreeSelection_Ref_Ptr->signal_changed().connect(
        sigc::mem_fun(*this, &GUI_Presentation::Handle_SimStats_ListView_SerList_Row_Select));

    Bind_Drawing_Area_To_API(Widget_Ids::SimStats_Draw_Area, SimStats_Drawing_Area_Ptr);

//  Setup the Notebook tab and record its Id
    NoteBook_Tab_Ids.SimStats_Tab_Id =
    SimSoup_Main_Notebook_Ptr->append_page(*SimStats_Main_Panes_Ptr, "Simulation Statistics");
}
//----------------------------------------------------------------------------------------------------------------

void GUI_Presentation::Setup_SysMon_Tab()

//  Create the System Monitor ListView

{
    CREATE_OBJECT(Gtk::ScrolledWindow, SysMon_ScrolledWindow_Ptr = new  Gtk::ScrolledWindow);
    CREATE_OBJECT(Gtk::TreeView, SysMon_ListView_Ptr = new Gtk::TreeView);
    SysMon_ScrolledWindow_Ptr->add(*SysMon_ListView_Ptr);

//  Setup the ListView and the ListStore

    SysMon_ListView_Ptr->append_column("Class For Diagnostis Report", SysMon_ListStore_Cols.Class_Col); 
    SysMon_ListView_Ptr->append_column("Object Count", SysMon_ListStore_Cols.Object_Count_Col); 
    SysMon_ListView_Ptr->append_column("Creations", SysMon_ListStore_Cols.Creations_Col); 
    SysMon_ListView_Ptr->append_column("Deletions", SysMon_ListStore_Cols.Deletions_Col); 
 
    for(int ILoop = 0; ; ILoop++)
    {
        Gtk::TreeView::Column* Col_Ptr = SysMon_ListView_Ptr->get_column(ILoop);
        if (Col_Ptr == 0) break;
        Col_Ptr->set_resizable();
    }

    SysMon_ListStore_RefPtr = Gtk::ListStore::create(SysMon_ListStore_Cols);
    SysMon_ListView_Ptr->set_model(SysMon_ListStore_RefPtr);

    Bind_ListStore_To_API(Widget_Ids::SysMon_ListStore, SysMon_ListStore_RefPtr);

//  Setup the notebook tab and record its Id
    NoteBook_Tab_Ids.SysMon_Tab_Id =
    SimSoup_Main_Notebook_Ptr->append_page(*SysMon_ScrolledWindow_Ptr, "");

//    SimSoup_Main_Notebook_Ptr->signal_switch_page().connect(
//        sigc::mem_fun(*this, &GUI_Presentation::Handle_Main_Notebook_Switch_Page));
}
//-----------------------------------------------------------------------------------------------------------------

void GUI_Presentation::Start()

//  Start the User Interface Presentation Layer

{
    TRACE("GUI_Presentation::Start");
    Gtk::Main  Gtk_Main_Instance(0, 0);

//  Setup the singleton Colour object

    Colour();

//  Setup the pre-defined colours

    List_Of_Colours.push_back(Colour::Blue());
    List_Of_Colours.push_back(Colour::Red());
    List_Of_Colours.push_back(Colour::Green());
    List_Of_Colours.push_back(Colour::Black());
    List_Of_Colours.push_back(Colour::White());
    List_Of_Colours.push_back(Colour::LimeGreen());
    List_Of_Colours.push_back(Colour::Maroon());

//  Setup the Main Window

    CREATE_OBJECT(Gtk::Window,SimSoup_Main_Window_Ptr = new Gtk::Window);
    SimSoup_Main_Window_Ptr->set_title("SimSoup");
    CREATE_OBJECT(Gtk::VBox, SimSoup_Main_VBox_Ptr = new Gtk::VBox);
    CREATE_OBJECT(Gtk::Notebook, SimSoup_Main_Notebook_Ptr = new Gtk::Notebook);

    SimSoup_Main_Window_Ptr->add(*SimSoup_Main_VBox_Ptr);

    Setup_Main_Menu();

//  Setup the Notebook that holds the tabs for the application display

    SimSoup_Main_VBox_Ptr->pack_start(*SimSoup_Main_Notebook_Ptr);

    SimSoup_Main_Window_Ptr->maximize();

//---------------- Setup the Notebook Tabs ------------------------------------------------------------------------
   
    Setup_Simulation_Control_Tab();
    Setup_Chem_And_Reactor_Tab();
    Setup_SimStats_Tab();
    Setup_Help_Tab();
    Setup_SysMon_Tab();

//----------------------------------------------------------------------------------------------------------------

//  Show the user interface 

    SimSoup_Main_Window_Ptr->show_all();
    SimSoup_Main_Notebook_Ptr->set_current_page(NoteBook_Tab_Ids.SimControl_Tab_Id);

//  Setup handler for switching tabs (need to do this after showing tab)
    SimSoup_Main_Notebook_Ptr->signal_switch_page().connect(
        sigc::mem_fun(*this, &GUI_Presentation::Handle_Main_Notebook_Switch_Page));

    int Width;
    int Height;
    SimSoup_Main_Window_Ptr->get_size(Width, Height);
    SimStats_Main_Panes_Ptr->set_position(Width/5);

//================  Start the main Gtk loop ======================================================================
    Gtk_Main_Instance.run();
//================================================================================================================
}
//----------------------------------------------------------------------------------------------------------------

GUI_Presentation::GUI_Presentation()

{

}
//----------------------------------------------------------------------------------------------------------------

void GUI_Presentation::Text_Out(Widget_Ids::Drawing_Area_Id  Drawing_Area_Spec,
                                int X, int Y, string Text)

//  Wtite the specified text to the specified Drawing Area

{
    List_Of_Drawing_Areas[Drawing_Area_Spec]->Text_Out(X, Y, Text);
}
//----------------------------------------------------------------------------------------------------------------

void GUI_Presentation::Update_Selected_Row(Widget_Ids::ListView_Id Id, 
                                           list<string>& List_Of_Col_Texts, 
                                           void* Data_Ptr)

{
    switch (Id)
    {
        case Widget_Ids::SimStats_Groups_ListStore:
            break;

        case Widget_Ids::SimStats_Series_ListStore:
            BEGIN
                Gtk::TreeModel::iterator Iter = List_Of_ListViews[Id]->get_selection()->get_selected();
                Gtk::TreeModel::Row row = *Iter;
    
                if (not List_Of_Col_Texts.empty())
                THEN
                    row[SimStats_ListStore_Series_Cols.Series_Name_Col] = List_Of_Col_Texts.front();
                    List_Of_Col_Texts.pop_front();
                ENDIF
                if (not List_Of_Col_Texts.empty())
                THEN
                    row[SimStats_ListStore_Series_Cols.Plot_Flag_Col] = List_Of_Col_Texts.front();
                    List_Of_Col_Texts.pop_front();
                ENDIF
                row[SimStats_ListStore_Series_Cols.Indexed_Series_Ptr_Col] = Data_Ptr;  // Linked data
    
                Assert(List_Of_Col_Texts.empty(), "GUI_Presentation::Add_Row - Too many texts for SimStats_Seriess_ListStore");
            END
            break;
        default:
            Assert(true == false, "GUI_Presentation::Add_Row - Invalid ListStore_Id");
            break;
    }     
}
