/****************************************************************************
*
*	  Header Filename:  Trace.h
*	  Module Name:      Trace
*	  Description:      Trace utility for debugging
*
*	  Author:           Chris Gordon-Smith
*
*     Copyright (C) 2008 Chris Gordon-Smith
*     Email: See http://www.simsoup.info/
*
*     This program is free software; you can redistribute it and/or modify it
*     under the terms of the GNU General Public License as published by the
*     Free Software Foundation; either version 2 of the License, or (at your
*     option) any later version.
*
*     This program is distributed in the hope that it will be useful, but
*     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
*     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
*     for more details.
*
*     You should have received a copy of the GNU General Public License along
*     with this program; if not, write to the Free Software Foundation, Inc.,
*     59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*
*     See the file Licence.txt for a copy of the GNU General Public License.
*
*      Amendment History:- 
*
*              Version	  Date                        Update Description
*              =======    ===========                 ==================
*
*
*
*
****************************************************************************/

#ifndef Trace_H
#define Trace_H

class   Trace_  // Underscore used to avoid accidental use
                // Trace_ should always be called using Macro TRACE

//	Based on example in C++ Gotchas (Stephen C Dewhurst):	
//	Gotcha 67 - Failure to employ Resource Acquistion Is Initialization

//  Note that it is assumed that Trace assumes that the text to 
//  which Msg_Ptr points remains valid for the lifetime of the Trace object

{
    public:
        Trace_(const char* Msg_Ptr): Message_Ptr(Msg_Ptr)
        {
            if (Trace_Flag)
            THEN       
                Indent_String += Two_Blanks;
                Logger::Debug(string(Indent_String + Enter_Text + Message_Ptr).c_str());
            ENDIF
        }   
//----------------------------------------------------------------------------------------------------------------
        ~Trace_()
        {
            if (Trace_Flag)
            THEN
                Logger::Debug(string(Indent_String + Leave_Text + Message_Ptr).c_str());
                Indent_String.erase(0,2);
            ENDIF
        }
//----------------------------------------------------------------------------------------------------------------

        static  void TraceVal(const char* Value_Name, const std::string  Value)
        {
            Logger::Debug(string(Indent_String + " VALUE: " + string(Value_Name) + " = " + Value).c_str());
        }

        static void Enable() {
                                    Logger::Debug("===============  Trace Enabled ===============");
                                    Trace_Flag = true;};
        static void Disable() {
                                    Logger::Debug("===============  Trace Disabled ==============");
                                    Trace_Flag = false;};
        static bool Is_Enabled() {return Trace_Flag;};

	        
    private:
        const char*  Message_Ptr;
        static std::string  Enter_Text;
        static std::string  Leave_Text;
        static std::string  Indent_String;
        static std::string  Two_Blanks;
        static bool   Trace_Flag;
};

//  Trace and Trace Control Macros  - Use to log entry and exit to function

#if     defined (ENABLE_TRACE_GLOBAL) && defined (ENABLE_TRACE_FILE)
#define TRACE(Trace_Text) \
Trace_  Trace_Object_With_Name_Extremely_Unlikely_To_Clash_With_Non_Debug_Code_(Trace_Text);
#define TRACEVAL(Value) \
if (Trace_::Is_Enabled()) Trace_::TraceVal((#Value), To_String(Value));
#define START_TRACE  Trace_::Enable();
#define STOP_TRACE   Trace_::Disable();
#else
#define TRACE(Trace_Text) ;
#define TRACEVAL(Value) ;
#define START_TRACE  ;
#define STOP_TRACE   ;
#endif  

#endif  //Trace_H



