/****************************************************************************
*
*     Module Filename:  main.cpp
*     Module Name:      Main
*     Description:      SimSoup main program              
*     Author:           Chris Gordon-Smith
*
*     Copyright (C) 2005, 2006, 2008 Chris Gordon-Smith
*     Email: See http://www.simsoup.info/
*
*     This program is free software; you can redistribute it and/or modify it
*     under the terms of the GNU General Public License Version 2, as published 
*     by the Free Software Foundation.
*
*     This program is distributed in the hope that it will be useful, but
*     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
*     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
*     for more details.
*
*     You should have received a copy of the GNU General Public License along
*     with this program; if not, write to the Free Software Foundation, Inc.,
*     59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*
*     See the file Licence.txt for a copy of the GNU General Public License.
*     Amendment History:-
*
*         Version    Date                 Update Description
*         =======    ===========          ==================
*
*
*
*
****************************************************************************/

#define  ENABLE_TRACE_FILE 

#include    "globldef.h"
#include    "Library_Headers.h"

//--------------  GUI_Presentation  Subsystem  --------------------------------------------------------------------
#include    "GUI_Presentation_API.h"
//----------------------------------------------------------------------------------------------------------------

//--------------  User Interface Manager ------------------------------------------------------------------------
#include  "Form_Main.h"
#include  "User_Actions_API.h"
//----------------------------------------------------------------------------------------------------------------

//---------------- Kernel Manager Subsystem  ---------------------------------------------------------------------
#include  "Kernel_Manager.h"
//----------------------------------------------------------------------------------------------------------------

//---------------- Persistent Data Manager -----------------------------------------------------------------------
#include  "Persistent_Data_Manager.h"
//----------------------------------------------------------------------------------------------------------------

//----------------  Statistics Subststem  ------------------------------------------------------------------------
#include  "Simulation_Statistics.h"
//----------------------------------------------------------------------------------------------------------------

int main(int argc, char* argv[])

//  Start the SimSoup system

{
    System_Monitor::Make();  //  Create the System Monitor. Must do this 
                             //  before any other CREATE_OBJECT
                             //  because CREATE_OBJECT macro expects 
                             //  System_Monitor to exist

    Logger::Make();
//    START_TRACE  // Cannot do this until Logger exists
	
    Persistent_Data_Manager::Start();

    Simulation_Statistics::Make();  
	
    Kernel_Manager::Start();    //  Start the SimSoup Kernel

    MainForm::Make();           //  Setup the User Interface Manager

    User_Actions_API::Initialise_API();  // Setup the User Action API 
                                         // for the User Interface Manager
    GUI_Presentation_API::Start(); //  Setup the Presentation Layer
                                   //  and start the Gtk main loop

    return 0;
}
